@@ -80,7 +80,7 @@ static QString findExecutable(const QString &fileName)
         if (QFileInfo::exists(fullPath))
             return QDir::cleanPath(fullPath);
     }
-    return QString();
+    return {};
 }
 
 static QString qsystem(const QString &exe, const QStringList &args = QStringList())
@@ -92,7 +92,7 @@ static QString qsystem(const QString &exe, const QStringList &args = QStringList
         throw qbs::ErrorInfo(Tr::tr("Failed to start compiler '%1': %2")
                              .arg(exe, p.errorString()));
     }
-    if (!p.waitForFinished() || p.exitCode() != 0)
+    if (!p.waitForFinished(-1) || p.exitCode() != 0)
         throw qbs::ErrorInfo(Tr::tr("Failed to run compiler '%1': %2").arg(exe, p.errorString()));
     return QString::fromLocal8Bit(p.readAll());
 }
@@ -100,41 +100,67 @@ static QString qsystem(const QString &exe, const QStringList &args = QStringList
 static QStringList validMinGWMachines()
 {
     // List of MinGW machine names (gcc -dumpmachine) recognized by Qbs
-    return QStringList()
-            << QLatin1String("mingw32") << QLatin1String("mingw64")
-            << QLatin1String("i686-w64-mingw32") << QLatin1String("x86_64-w64-mingw32")
-            << QLatin1String("i686-w64-mingw32.shared") << QLatin1String("x86_64-w64-mingw32.shared")
-            << QLatin1String("i686-w64-mingw32.static") << QLatin1String("x86_64-w64-mingw32.static")
-            << QLatin1String("i586-mingw32msvc") << QLatin1String("amd64-mingw32msvc");
+    return {QStringLiteral("mingw32"), QStringLiteral("mingw64"),
+            QStringLiteral("i686-w64-mingw32"), QStringLiteral("x86_64-w64-mingw32"),
+            QStringLiteral("i686-w64-mingw32.shared"), QStringLiteral("x86_64-w64-mingw32.shared"),
+            QStringLiteral("i686-w64-mingw32.static"), QStringLiteral("x86_64-w64-mingw32.static"),
+            QStringLiteral("i586-mingw32msvc"), QStringLiteral("amd64-mingw32msvc")};
+}
+
+static QStringList knownIarCompilerNames()
+{
+    return {QStringLiteral("icc8051"), QStringLiteral("iccarm"), QStringLiteral("iccavr")};
+}
+
+static bool isIarCompiler(const QString &compilerName)
+{
+    return Internal::any_of(knownIarCompilerNames(), [compilerName](const QString &knownName) {
+        return compilerName.contains(knownName);
+    });
+}
+
+static QStringList knownKeilCompilerNames()
+{
+    return {QStringLiteral("c51"), QStringLiteral("armcc")};
+}
+
+static bool isKeilCompiler(const QString &compilerName)
+{
+    return Internal::any_of(knownKeilCompilerNames(), [compilerName](const QString &knownName) {
+        return compilerName.contains(knownName);
+    });
 }
 
 static QStringList toolchainTypeFromCompilerName(const QString &compilerName)
 {
     if (compilerName == QLatin1String("cl.exe"))
-        return canonicalToolchain(QLatin1String("msvc"));
-    const auto types = { QLatin1String("clang"), QLatin1String("llvm"),
-                         QLatin1String("mingw"), QLatin1String("gcc") };
-    for (const QString &type : types) {
+        return canonicalToolchain(QStringLiteral("msvc"));
+    const auto types = { QStringLiteral("clang"), QStringLiteral("llvm"),
+                         QStringLiteral("mingw"), QStringLiteral("gcc") };
+    for (const auto &type : types) {
         if (compilerName.contains(type))
             return canonicalToolchain(type);
     }
     if (compilerName == QLatin1String("g++"))
-        return canonicalToolchain(QLatin1String("gcc"));
-    return QStringList();
+        return canonicalToolchain(QStringLiteral("gcc"));
+    if (isIarCompiler(compilerName))
+        return canonicalToolchain(QStringLiteral("iar"));
+    if (isKeilCompiler(compilerName))
+        return canonicalToolchain(QStringLiteral("keil"));
+    return {};
 }
 
 static QString gccMachineName(const QString &compilerFilePath)
 {
-    return qsystem(compilerFilePath, QStringList() << QLatin1String("-dumpmachine")).trimmed();
+    return qsystem(compilerFilePath, QStringList() << QStringLiteral("-dumpmachine")).trimmed();
 }
 
 static QStringList standardCompilerFileNames()
 {
-    return QStringList()
-            << HostOsInfo::appendExecutableSuffix(QStringLiteral("gcc"))
-            << HostOsInfo::appendExecutableSuffix(QStringLiteral("g++"))
-            << HostOsInfo::appendExecutableSuffix(QStringLiteral("clang"))
-            << HostOsInfo::appendExecutableSuffix(QStringLiteral("clang++"));
+    return {HostOsInfo::appendExecutableSuffix(QStringLiteral("gcc")),
+            HostOsInfo::appendExecutableSuffix(QStringLiteral("g++")),
+            HostOsInfo::appendExecutableSuffix(QStringLiteral("clang")),
+            HostOsInfo::appendExecutableSuffix(QStringLiteral("clang++"))};
 }
 
 static void setCommonProperties(Profile &profile, const QString &compilerFilePath,
@@ -148,15 +174,15 @@ static void setCommonProperties(Profile &profile, const QString &compilerFilePat
 
     const QString prefix = compilerName.left(compilerName.lastIndexOf(QLatin1Char('-')) + 1);
     if (!prefix.isEmpty())
-        profile.setValue(QLatin1String("cpp.toolchainPrefix"), prefix);
+        profile.setValue(QStringLiteral("cpp.toolchainPrefix"), prefix);
 
-    profile.setValue(QLatin1String("cpp.toolchainInstallPath"), cfi.absolutePath());
-    profile.setValue(QLatin1String("qbs.toolchain"), toolchainTypes);
+    profile.setValue(QStringLiteral("cpp.toolchainInstallPath"), cfi.absolutePath());
+    profile.setValue(QStringLiteral("qbs.toolchain"), toolchainTypes);
 
     const QString suffix = compilerName.right(compilerName.size() - prefix.size());
     if (!standardCompilerFileNames().contains(suffix))
         qWarning("%s", qPrintable(
-                     QString::fromLatin1("'%1' is not a standard compiler file name; "
+                     QStringLiteral("'%1' is not a standard compiler file name; "
                                             "you must set the cpp.cCompilerName and "
                                             "cpp.cxxCompilerName properties of this profile "
                                             "manually").arg(compilerName)));
@@ -177,9 +203,8 @@ public:
             return;
         const QString toolFilePath = findExecutable(toolFileName);
         if (toolFilePath.isEmpty()) {
-            qWarning("%s", qPrintable(
-                         QString(QLatin1String("'%1' exists neither in '%2' nor in PATH."))
-                            .arg(toolFileName, m_compilerDirPath)));
+            qWarning("%s", qPrintable(QStringLiteral("'%1' exists neither in '%2' nor in PATH.")
+                     .arg(toolFileName, m_compilerDirPath)));
         }
         m_profile->setValue(propertyName, toolFilePath);
     }
@@ -223,20 +248,84 @@ static Profile createGccProfile(const QString &compilerFilePath, Settings *setti
     const ToolPathSetup toolPathSetup(&profile, compilerDirPath,
                                       profile.value(QStringLiteral("cpp.toolchainPrefix"))
                                       .toString());
-    toolPathSetup.apply(QLatin1String("ar"), QLatin1String("cpp.archiverPath"));
-    toolPathSetup.apply(QLatin1String("as"), QLatin1String("cpp.assemblerPath"));
-    toolPathSetup.apply(QLatin1String("nm"), QLatin1String("cpp.nmPath"));
+    toolPathSetup.apply(QStringLiteral("ar"), QStringLiteral("cpp.archiverPath"));
+    toolPathSetup.apply(QStringLiteral("as"), QStringLiteral("cpp.assemblerPath"));
+    toolPathSetup.apply(QStringLiteral("nm"), QStringLiteral("cpp.nmPath"));
     if (doesProfileTargetOS(profile, QStringLiteral("darwin")))
-        toolPathSetup.apply(QLatin1String("dsymutil"), QLatin1String("cpp.dsymutilPath"));
+        toolPathSetup.apply(QStringLiteral("dsymutil"), QStringLiteral("cpp.dsymutilPath"));
     else
-        toolPathSetup.apply(QLatin1String("objcopy"), QLatin1String("cpp.objcopyPath"));
-    toolPathSetup.apply(QLatin1String("strip"), QLatin1String("cpp.stripPath"));
+        toolPathSetup.apply(QStringLiteral("objcopy"), QStringLiteral("cpp.objcopyPath"));
+    toolPathSetup.apply(QStringLiteral("strip"), QStringLiteral("cpp.stripPath"));
 
     qStdout << Tr::tr("Profile '%1' created for '%2'.").arg(profile.name(), compilerFilePath)
             << endl;
     return profile;
 }
 
+static QString guessIarArchitecture(const QFileInfo &compiler)
+{
+    const auto baseName = compiler.baseName();
+    if (baseName == QLatin1String("icc8051"))
+        return QStringLiteral("mcs51");
+    if (baseName == QLatin1String("iccarm"))
+        return QStringLiteral("arm");
+    if (baseName == QLatin1String("iccavr"))
+        return QStringLiteral("avr");
+    return {};
+}
+
+static Profile createIarProfile(const QFileInfo &compiler, Settings *settings,
+                                QString profileName = QString())
+{
+    const QString architecture = guessIarArchitecture(compiler);
+
+    // In case the profile is auto-detected.
+    if (profileName.isEmpty())
+        profileName = QLatin1String("iar-") + architecture;
+
+    Profile profile(profileName, settings);
+    profile.setValue(QLatin1String("cpp.toolchainInstallPath"), compiler.absolutePath());
+    profile.setValue(QLatin1String("qbs.toolchainType"), QLatin1String("iar"));
+    if (!architecture.isEmpty())
+        profile.setValue(QLatin1String("qbs.architecture"), architecture);
+
+    qStdout << Tr::tr("Profile '%1' created for '%2'.").arg(
+                   profile.name(), compiler.absoluteFilePath())
+            << endl;
+    return profile;
+}
+
+static QString guessKeilArchitecture(const QFileInfo &compiler)
+{
+    const auto baseName = compiler.baseName();
+    if (baseName == QLatin1String("c51"))
+        return QStringLiteral("mcs51");
+    if (baseName == QLatin1String("armcc"))
+        return QStringLiteral("arm");
+    return {};
+}
+
+static Profile createKeilProfile(const QFileInfo &compiler, Settings *settings,
+                                 QString profileName = QString())
+{
+    const QString architecture = guessKeilArchitecture(compiler);
+
+    // In case the profile is auto-detected.
+    if (profileName.isEmpty())
+        profileName = QLatin1String("keil-") + architecture;
+
+    Profile profile(profileName, settings);
+    profile.setValue(QStringLiteral("cpp.toolchainInstallPath"), compiler.absolutePath());
+    profile.setValue(QStringLiteral("qbs.toolchainType"), QStringLiteral("keil"));
+    if (!architecture.isEmpty())
+        profile.setValue(QStringLiteral("qbs.architecture"), architecture);
+
+    qStdout << Tr::tr("Profile '%1' created for '%2'.").arg(
+                   profile.name(), compiler.absoluteFilePath())
+            << endl;
+    return profile;
+}
+
 static void gccProbe(Settings *settings, QList<Profile> &profiles, const QString &compilerName)
 {
     qStdout << Tr::tr("Trying to detect %1...").arg(compilerName) << endl;
@@ -258,7 +347,7 @@ static void mingwProbe(Settings *settings, QList<Profile> &profiles)
     // List of possible compiler binary names for this platform
     QStringList compilerNames;
     if (HostOsInfo::isWindowsHost()) {
-        compilerNames << QLatin1String("gcc");
+        compilerNames << QStringLiteral("gcc");
     } else {
         const auto machineNames = validMinGWMachines();
         for (const QString &machineName : machineNames) {
@@ -271,18 +360,54 @@ static void mingwProbe(Settings *settings, QList<Profile> &profiles)
                 = findExecutable(HostOsInfo::appendExecutableSuffix(compilerName));
         if (!gccPath.isEmpty())
             profiles.push_back(createGccProfile(gccPath, settings,
-                                                canonicalToolchain(QLatin1String("mingw"))));
+                                                canonicalToolchain(QStringLiteral("mingw"))));
     }
 }
 
+static void iarProbe(Settings *settings, QList<Profile> &profiles)
+{
+    qStdout << Tr::tr("Trying to detect IAR toolchains...") << endl;
+
+    bool isFound = false;
+    for (const QString &compilerName : knownIarCompilerNames()) {
+        const QString iarPath = findExecutable(HostOsInfo::appendExecutableSuffix(compilerName));
+        if (!iarPath.isEmpty()) {
+            const auto profile = createIarProfile(iarPath, settings);
+            profiles.push_back(profile);
+            isFound = true;
+        }
+    }
+
+    if (!isFound)
+        qStdout << Tr::tr("No IAR toolchains found.") << endl;
+}
+
+static void keilProbe(Settings *settings, QList<Profile> &profiles)
+{
+    qStdout << Tr::tr("Trying to detect KEIL toolchains...") << endl;
+
+    bool isFound = false;
+    for (const QString &compilerName : knownKeilCompilerNames()) {
+        const QString keilPath = findExecutable(HostOsInfo::appendExecutableSuffix(compilerName));
+        if (!keilPath.isEmpty()) {
+            const auto profile = createKeilProfile(keilPath, settings);
+            profiles.push_back(profile);
+            isFound = true;
+        }
+    }
+
+    if (!isFound)
+        qStdout << Tr::tr("No KEIL toolchains found.") << endl;
+}
+
 void probe(Settings *settings)
 {
     QList<Profile> profiles;
     if (HostOsInfo::isWindowsHost()) {
         msvcProbe(settings, profiles);
     } else {
-        gccProbe(settings, profiles, QLatin1String("gcc"));
-        gccProbe(settings, profiles, QLatin1String("clang"));
+        gccProbe(settings, profiles, QStringLiteral("gcc"));
+        gccProbe(settings, profiles, QStringLiteral("clang"));
 
         if (HostOsInfo::isMacosHost()) {
             xcodeProbe(settings, profiles);
@@ -290,13 +415,15 @@ void probe(Settings *settings)
     }
 
     mingwProbe(settings, profiles);
+    iarProbe(settings, profiles);
+    keilProbe(settings, profiles);
 
     if (profiles.empty()) {
         qStderr << Tr::tr("Could not detect any toolchains. No profile created.") << endl;
     } else if (profiles.size() == 1 && settings->defaultProfile().isEmpty()) {
         const QString profileName = profiles.front().name();
         qStdout << Tr::tr("Making profile '%1' the default.").arg(profileName) << endl;
-        settings->setValue(QLatin1String("defaultProfile"), profileName);
+        settings->setValue(QStringLiteral("defaultProfile"), profileName);
     }
 }
 
@@ -322,6 +449,10 @@ void createProfile(const QString &profileName, const QString &toolchainType,
         createMsvcProfile(profileName, compiler.absoluteFilePath(), settings);
     else if (toolchainTypes.contains(QLatin1String("gcc")))
         createGccProfile(compiler.absoluteFilePath(), settings, toolchainTypes, profileName);
+    else if (toolchainTypes.contains(QLatin1String("iar")))
+        createIarProfile(compiler, settings, profileName);
+    else if (toolchainTypes.contains(QLatin1String("keil")))
+        createKeilProfile(compiler, settings, profileName);
     else
         throw qbs::ErrorInfo(Tr::tr("Cannot create profile: Unknown toolchain type."));
 }
