@@ -47,13 +47,14 @@
 #include <tools/profile.h>
 #include <tools/settings.h>
 
-#include <QStringList>
-#include <QProcess>
-#include <QByteArray>
-#include <QFileInfo>
-#include <QDir>
-#include <QSettings>
-#include <QRegularExpression>
+#include <QtCore/qstringlist.h>
+#include <QtCore/qprocess.h>
+#include <QtCore/qbytearray.h>
+#include <QtCore/qfileinfo.h>
+#include <QtCore/qdir.h>
+#include <QtCore/qsettings.h>
+
+#include <regex>
 
 using namespace qbs;
 using Internal::Tr;
@@ -61,8 +62,8 @@ using Internal::Tr;
 namespace {
 static const QString defaultDeveloperPath =
         QStringLiteral("/Applications/Xcode.app/Contents/Developer");
-static const QRegularExpression defaultDeveloperPathRegex(
-        QStringLiteral("^/Applications/Xcode([a-zA-Z0-9 _-]+)\\.app/Contents/Developer$"));
+static const std::regex defaultDeveloperPathRegex(
+        "^/Applications/Xcode([a-zA-Z0-9 _-]+)\\.app/Contents/Developer$");
 
 class XcodeProbe
 {
@@ -90,7 +91,7 @@ bool XcodeProbe::addDeveloperPath(const QString &path)
         return false;
     if (developerPaths.contains(path))
         return false;
-    developerPaths.append(path);
+    developerPaths.push_back(path);
     qbsInfo() << Tr::tr("Added developer path %1").arg(path);
     return true;
 }
@@ -98,10 +99,10 @@ bool XcodeProbe::addDeveloperPath(const QString &path)
 void XcodeProbe::detectDeveloperPaths()
 {
     QProcess selectedXcode;
-    QString program = QLatin1String("/usr/bin/xcode-select");
-    QStringList arguments(QLatin1String("--print-path"));
+    QString program = QStringLiteral("/usr/bin/xcode-select");
+    QStringList arguments(QStringLiteral("--print-path"));
     selectedXcode.start(program, arguments, QProcess::ReadOnly);
-    if (!selectedXcode.waitForFinished() || selectedXcode.exitCode()) {
+    if (!selectedXcode.waitForFinished(-1) || selectedXcode.exitCode()) {
         qbsInfo() << Tr::tr("Could not detect selected Xcode with /usr/bin/xcode-select");
     } else {
         QString path = QString::fromLocal8Bit(selectedXcode.readAllStandardOutput());
@@ -110,10 +111,10 @@ void XcodeProbe::detectDeveloperPaths()
     addDeveloperPath(defaultDeveloperPath);
 
     QProcess launchServices;
-    program = QLatin1String("/usr/bin/mdfind");
-    arguments = QStringList(QLatin1String("kMDItemCFBundleIdentifier == 'com.apple.dt.Xcode'"));
+    program = QStringLiteral("/usr/bin/mdfind");
+    arguments = QStringList(QStringLiteral("kMDItemCFBundleIdentifier == 'com.apple.dt.Xcode'"));
     launchServices.start(program, arguments, QProcess::ReadOnly);
-    if (!launchServices.waitForFinished() || launchServices.exitCode()) {
+    if (!launchServices.waitForFinished(-1) || launchServices.exitCode()) {
         qbsInfo() << Tr::tr("Could not detect additional Xcode installations with /usr/bin/mdfind");
     } else {
         for (const QString &path : QString::fromLocal8Bit(launchServices.readAllStandardOutput())
@@ -122,27 +123,23 @@ void XcodeProbe::detectDeveloperPaths()
     }
 }
 
-static QStringList targetOSList(const QString &applePlatformName)
+static QString targetOS(const QString &applePlatformName)
 {
-    QStringList targetOS;
-    if (applePlatformName == QStringLiteral("macosx")) {
-        targetOS << QStringLiteral("macos");
-    } else if (applePlatformName == QStringLiteral("iphoneos")) {
-        targetOS << QStringLiteral("ios");
-    } else if (applePlatformName == QStringLiteral("iphonesimulator")) {
-        targetOS << QStringLiteral("ios") << QStringLiteral("ios-simulator");
-    } else if (applePlatformName == QStringLiteral("appletvos")) {
-        targetOS << QStringLiteral("tvos");
-    } else if (applePlatformName == QStringLiteral("appletvsimulator")) {
-        targetOS << QStringLiteral("tvos") << QStringLiteral("tvos-simulator");
-    } else if (applePlatformName == QStringLiteral("watchos")) {
-        targetOS << QStringLiteral("watchos");
-    } else if (applePlatformName == QStringLiteral("watchsimulator")) {
-        targetOS << QStringLiteral("watchos") << QStringLiteral("watchos-simulator");
-    }
-
-    targetOS << QStringLiteral("darwin") << QStringLiteral("bsd") << QStringLiteral("unix");
-    return targetOS;
+    if (applePlatformName == QStringLiteral("macosx"))
+        return QStringLiteral("macos");
+    if (applePlatformName == QStringLiteral("iphoneos"))
+        return QStringLiteral("ios");
+    if (applePlatformName == QStringLiteral("iphonesimulator"))
+        return QStringLiteral("ios-simulator");
+    if (applePlatformName == QStringLiteral("appletvos"))
+        return QStringLiteral("tvos");
+    if (applePlatformName == QStringLiteral("appletvsimulator"))
+        return QStringLiteral("tvos-simulator");
+    if (applePlatformName == QStringLiteral("watchos"))
+        return QStringLiteral("watchos");
+    if (applePlatformName == QStringLiteral("watchsimulator"))
+        return QStringLiteral("watchos-simulator");
+    return {};
 }
 
 static QStringList archList(const QString &applePlatformName)
@@ -159,7 +156,7 @@ static QStringList archList(const QString &applePlatformName)
     } else if (applePlatformName == QStringLiteral("iphoneos")
                || applePlatformName == QStringLiteral("appletvos")) {
         if (applePlatformName != QStringLiteral("appletvos"))
-            archs << QStringLiteral("armv7");
+            archs << QStringLiteral("armv7a");
         archs << QStringLiteral("arm64");
     } else if (applePlatformName == QStringLiteral("watchos")) {
         archs << QStringLiteral("armv7k");
@@ -175,13 +172,13 @@ void XcodeProbe::setupDefaultToolchains(const QString &devPath, const QString &x
     Profile installationProfile(xcodeName, settings);
     installationProfile.removeProfile();
     installationProfile.setValue(QStringLiteral("qbs.toolchain"), QStringList()
-                                 << QLatin1String("xcode")
-                                 << QLatin1String("clang")
-                                 << QLatin1String("llvm")
-                                 << QLatin1String("gcc"));
+                                 << QStringLiteral("xcode")
+                                 << QStringLiteral("clang")
+                                 << QStringLiteral("llvm")
+                                 << QStringLiteral("gcc"));
     if (devPath != defaultDeveloperPath)
         installationProfile.setValue(QStringLiteral("xcode.developerPath"), devPath);
-    profiles << installationProfile;
+    profiles.push_back(installationProfile);
 
     QStringList platforms;
     platforms << QStringLiteral("macosx")
@@ -195,8 +192,8 @@ void XcodeProbe::setupDefaultToolchains(const QString &devPath, const QString &x
         Profile platformProfile(xcodeName + QLatin1Char('-') + platform, settings);
         platformProfile.removeProfile();
         platformProfile.setBaseProfile(installationProfile.name());
-        platformProfile.setValue(QStringLiteral("qbs.targetOS"), targetOSList(platform));
-        profiles << platformProfile;
+        platformProfile.setValue(QStringLiteral("qbs.targetPlatform"), targetOS(platform));
+        profiles.push_back(platformProfile);
 
         for (const QString &arch : archList(platform)) {
             Profile archProfile(xcodeName + QLatin1Char('-') + platform + QLatin1Char('-') + arch,
@@ -205,7 +202,7 @@ void XcodeProbe::setupDefaultToolchains(const QString &devPath, const QString &x
             archProfile.setBaseProfile(platformProfile.name());
             archProfile.setValue(QStringLiteral("qbs.architecture"),
                                  qbs::canonicalArchitecture(arch));
-            profiles << archProfile;
+            profiles.push_back(archProfile);
         }
     }
 }
@@ -215,13 +212,15 @@ void XcodeProbe::detectAll()
     int i = 1;
     detectDeveloperPaths();
     for (const QString &developerPath : developerPaths) {
-        QRegularExpressionMatch match(defaultDeveloperPathRegex.match(developerPath));
-        QString profileName = QLatin1String("xcode");
+        QString profileName = QStringLiteral("xcode");
         if (developerPath != defaultDeveloperPath) {
-            profileName += match.hasMatch()
-                    ? match.capturedTexts().value(1).toLower().replace(QLatin1Char(' '),
-                                                                       QLatin1Char('-'))
-                    : QString::number(i++);
+            const auto devPath = developerPath.toStdString();
+            std::smatch match;
+            if (std::regex_match(devPath, match, defaultDeveloperPathRegex))
+                profileName += QString::fromStdString(match[1]).toLower().replace(QLatin1Char(' '),
+                                                                                  QLatin1Char('-'));
+            else
+                profileName += QString::number(i++);
         }
         setupDefaultToolchains(developerPath, profileName);
     }
