@@ -66,7 +66,7 @@ class CommandLineFrontend : public QObject
     Q_OBJECT
 public:
     explicit CommandLineFrontend(const CommandLineParser &parser, Settings *settings,
-                                 QObject *parent = 0);
+                                 QObject *parent = nullptr);
     ~CommandLineFrontend();
 
     void cancel();
@@ -81,7 +81,7 @@ private:
     void handleProcessResultReport(const qbs::ProcessResult &result);
     void checkCancelStatus();
 
-    typedef QHash<Project, QList<ProductData> > ProductMap;
+    using ProductMap = QHash<Project, QList<ProductData>>;
     ProductMap productsToUse() const;
 
     bool resolvingMultipleProjects() const;
