@@ -44,11 +44,15 @@
 #include "buildgraphnode.h"
 #include "forward_decls.h"
 #include <language/filetags.h>
+#include <tools/dynamictypecheck.h>
 #include <tools/filetime.h>
 #include <tools/set.h>
 
 #include <QtCore/qstring.h>
 
+#include <utility>
+#include <vector>
+
 namespace qbs {
 namespace Internal {
 class Logger;
@@ -70,19 +74,32 @@ public:
     Artifact();
     ~Artifact();
 
-    Type type() const { return ArtifactNodeType; }
-    void accept(BuildGraphVisitor *visitor);
-    QString toString() const;
+    Type type() const override { return ArtifactNodeType; }
+    FileType fileType() const override { return FileTypeArtifact; }
+    void accept(BuildGraphVisitor *visitor) override;
+    QString toString() const override;
 
     void addFileTag(const FileTag &t);
     void removeFileTag(const FileTag &t);
     void setFileTags(const FileTags &newFileTags);
     const FileTags &fileTags() const { return m_fileTags; }
 
+    RuleNode *producer() const;
+
     ArtifactSet childrenAddedByScanner;
     Set<FileDependency *> fileDependencies;
     TransformerPtr transformer;
     PropertyMapPtr properties;
+    QString targetOfModule;
+
+    // The tags set directly via an Artifact item or an outputArtifacts script,
+    // not the result of file taggers or fileTagsFilter groups, nor the ones inherited from
+    // the product.
+    FileTags pureFileTags;
+
+    // The properties attached directly to an artifact in an Artifact item or outputArtifacts
+    // script.
+    std::vector<std::pair<QStringList, QVariant>> pureProperties;
 
     enum ArtifactType
     {
@@ -100,18 +117,29 @@ public:
     void initialize();
     const TypeFilter<Artifact> parentArtifacts() const;
     const TypeFilter<Artifact> childArtifacts() const;
-    void onChildDisconnected(BuildGraphNode *child);
+    void onChildDisconnected(BuildGraphNode *child) override;
+
+    bool isTargetOfModule() const { return !targetOfModule.isEmpty(); }
+
+    void load(PersistentPool &pool) override;
+    void store(PersistentPool &pool) override;
 
 private:
-    void load(PersistentPool &pool);
-    void store(PersistentPool &pool) const;
-
     FileTags m_fileTags;
 };
 
 template<> inline QString Set<Artifact *>::toString(Artifact * const &artifact) const
 {
-    return artifact->filePath();
+    return artifact ? artifact->filePath() : QStringLiteral("<null>");
+}
+template<> inline const void *uniqueAddress(const Artifact *a)
+{
+    return static_cast<const BuildGraphNode *>(a);
+}
+
+template<> inline bool hasDynamicType<Artifact>(const BuildGraphNode *n)
+{
+    return n->type() == BuildGraphNode::ArtifactNodeType;
 }
 
 // debugging helper
@@ -119,12 +147,12 @@ inline QString toString(Artifact::ArtifactType t)
 {
     switch (t) {
     case Artifact::SourceFile:
-        return QLatin1String("SourceFile");
+        return QStringLiteral("SourceFile");
     case Artifact::Generated:
-        return QLatin1String("Generated");
+        return QStringLiteral("Generated");
     case Artifact::Unknown:
     default:
-        return QLatin1String("Unknown");
+        return QStringLiteral("Unknown");
     }
 }
 
@@ -133,15 +161,15 @@ inline QString toString(BuildGraphNode::BuildState s)
 {
     switch (s) {
     case BuildGraphNode::Untouched:
-        return QLatin1String("Untouched");
+        return QStringLiteral("Untouched");
     case BuildGraphNode::Buildable:
-        return QLatin1String("Buildable");
+        return QStringLiteral("Buildable");
     case BuildGraphNode::Building:
-        return QLatin1String("Building");
+        return QStringLiteral("Building");
     case BuildGraphNode::Built:
-        return QLatin1String("Built");
+        return QStringLiteral("Built");
     default:
-        return QLatin1String("Unknown");
+        return QStringLiteral("Unknown");
     }
 }
 
