@@ -39,12 +39,14 @@
 #include "rulesapplicator.h"
 
 #include "buildgraph.h"
-#include "emptydirectoriesremover.h"
 #include "productbuilddata.h"
 #include "projectbuilddata.h"
 #include "qtmocscanner.h"
+#include "rulecommands.h"
+#include "rulenode.h"
 #include "rulesevaluationcontext.h"
 #include "transformer.h"
+#include "transformerchangetracking.h"
 
 #include <jsextensions/moduleproperties.h>
 #include <language/artifactproperties.h>
@@ -63,6 +65,7 @@
 #include <tools/qttools.h>
 #include <tools/stringconstants.h>
 
+#include <QtCore/qcryptographichash.h>
 #include <QtCore/qdir.h>
 #include <QtScript/qscriptvalueiterator.h>
 
@@ -72,8 +75,14 @@
 namespace qbs {
 namespace Internal {
 
-RulesApplicator::RulesApplicator(const ResolvedProductPtr &product, const Logger &logger)
+RulesApplicator::RulesApplicator(
+        const ResolvedProductPtr &product,
+        const std::unordered_map<QString, const ResolvedProduct *> &productsByName,
+        const std::unordered_map<QString, const ResolvedProject *> &projectsByName,
+        const Logger &logger)
     : m_product(product)
+    , m_productsByName(productsByName)
+    , m_projectsByName(projectsByName)
     , m_mocScanner(nullptr)
     , m_logger(logger)
 {
@@ -84,30 +93,33 @@ RulesApplicator::~RulesApplicator()
     delete m_mocScanner;
 }
 
-void RulesApplicator::applyRule(const RuleConstPtr &rule, const ArtifactSet &inputArtifacts)
+void RulesApplicator::applyRule(RuleNode *ruleNode, const ArtifactSet &inputArtifacts,
+                                const ArtifactSet &explicitlyDependsOn)
 {
-    if (inputArtifacts.empty() && rule->declaresInputs() && rule->requiresInputs)
-        return;
+    m_ruleNode = ruleNode;
+    m_rule = ruleNode->rule();
+    QBS_CHECK(!inputArtifacts.empty() || !m_rule->declaresInputs() || !m_rule->requiresInputs);
 
-    m_product->topLevelProject()->buildData->isDirty = true;
+    m_product->topLevelProject()->buildData->setDirty();
     m_createdArtifacts.clear();
     m_invalidatedArtifacts.clear();
+    m_removedArtifacts.clear();
+    m_explicitlyDependsOn = explicitlyDependsOn;
     RulesEvaluationContext::Scope s(evalContext().get());
 
-    m_rule = rule;
     m_completeInputSet = inputArtifacts;
-    if (rule->name == QLatin1String("QtCoreMocRule")) {
+    if (m_rule->name.startsWith(QLatin1String("QtCoreMocRule"))) {
         delete m_mocScanner;
         m_mocScanner = new QtMocScanner(m_product, scope());
     }
     QScriptValue prepareScriptContext = engine()->newObject();
     prepareScriptContext.setPrototype(engine()->globalObject());
-    PrepareScriptObserver observer(engine(), UnobserveMode::Enabled);
     setupScriptEngineForFile(engine(), m_rule->prepareScript.fileContext(), scope(),
                              ObserveMode::Enabled);
     setupScriptEngineForProduct(engine(), m_product.get(), m_rule->module.get(),
-                                prepareScriptContext, &observer, true);
+                                prepareScriptContext, true);
 
+    engine()->clearUsesIo();
     if (m_rule->multiplex) { // apply the rule once for a set of inputs
         doApply(inputArtifacts, prepareScriptContext);
     } else { // apply the rule once for each input
@@ -117,33 +129,38 @@ void RulesApplicator::applyRule(const RuleConstPtr &rule, const ArtifactSet &inp
             doApply(lst, prepareScriptContext);
         }
     }
+    if (engine()->usesIo())
+        m_ruleUsesIo = true;
 }
 
 void RulesApplicator::handleRemovedRuleOutputs(const ArtifactSet &inputArtifacts,
-        const ArtifactSet &outputArtifactsToRemove, const Logger &logger)
+        const ArtifactSet &outputArtifactsToRemove, QStringList &removedArtifacts,
+        const Logger &logger)
 {
     ArtifactSet artifactsToRemove;
     const TopLevelProject *project = nullptr;
     for (Artifact * const removedArtifact : outputArtifactsToRemove) {
-        qCDebug(lcBuildGraph) << "dynamic rule removed output artifact"
-                              << removedArtifact->toString();
+        qCDebug(lcBuildGraph).noquote() << "dynamic rule removed output artifact"
+                                        << removedArtifact->toString();
         if (!project)
             project = removedArtifact->product->topLevelProject();
         project->buildData->removeArtifactAndExclusiveDependents(removedArtifact, logger, true,
                                                                  &artifactsToRemove);
     }
-    // parents of removed artifacts must update their transformers
-    for (Artifact *removedArtifact : qAsConst(artifactsToRemove)) {
-        for (Artifact *parent : removedArtifact->parentArtifacts())
-            parent->product->registerArtifactWithChangedInputs(parent);
-    }
-    EmptyDirectoriesRemover(project, logger).removeEmptyParentDirectories(artifactsToRemove);
     for (Artifact * const artifact : qAsConst(artifactsToRemove)) {
         QBS_CHECK(!inputArtifacts.contains(artifact));
+        removedArtifacts << artifact->filePath();
         delete artifact;
     }
 }
 
+ArtifactSet RulesApplicator::collectAuxiliaryInputs(const Rule *rule,
+                                                    const ResolvedProduct *product)
+{
+    return collectAdditionalInputs(rule->auxiliaryInputs, rule, product,
+                                   CurrentProduct | Dependencies);
+}
+
 static void copyProperty(const QString &name, const QScriptValue &src, QScriptValue dst)
 {
     dst.setProperty(name, src.property(name));
@@ -163,18 +180,23 @@ static QStringList toStringList(const ArtifactSet &artifacts)
 void RulesApplicator::doApply(const ArtifactSet &inputArtifacts, QScriptValue &prepareScriptContext)
 {
     evalContext()->checkForCancelation();
+    for (const Artifact *inputArtifact : inputArtifacts)
+        QBS_CHECK(!inputArtifact->fileTags().intersects(m_rule->excludedInputs));
 
     qCDebug(lcBuildGraph) << "apply rule" << m_rule->toString()
                           << toStringList(inputArtifacts).join(QLatin1String(",\n            "));
 
-    QList<std::pair<const RuleArtifact *, Artifact *> > ruleArtifactArtifactMap;
+    QList<std::pair<const RuleArtifact *, OutputArtifactInfo>> ruleArtifactArtifactMap;
     QList<Artifact *> outputArtifacts;
 
     m_transformer = Transformer::create();
     m_transformer->rule = m_rule;
     m_transformer->inputs = inputArtifacts;
-    m_transformer->explicitlyDependsOn = collectExplicitlyDependsOn();
+    m_transformer->explicitlyDependsOn = m_explicitlyDependsOn;
     m_transformer->alwaysRun = m_rule->alwaysRun;
+    m_oldTransformer.reset();
+
+    engine()->clearRequestedProperties();
 
     // create the output artifacts from the set of input artifacts
     m_transformer->setupInputs(prepareScriptContext);
@@ -187,19 +209,37 @@ void RulesApplicator::doApply(const ArtifactSet &inputArtifacts, QScriptValue &p
     if (m_rule->isDynamic()) {
         outputArtifacts = runOutputArtifactsScript(inputArtifacts,
                     ScriptEngine::argumentList(Rule::argumentNamesForOutputArtifacts(), scope()));
-        ArtifactSet newOutputs = ArtifactSet::fromList(outputArtifacts);
-        const ArtifactSet oldOutputs = collectOldOutputArtifacts(inputArtifacts);
-        handleRemovedRuleOutputs(m_completeInputSet, oldOutputs - newOutputs, m_logger);
     } else {
         Set<QString> outputFilePaths;
-        for (const RuleArtifactConstPtr &ruleArtifact : qAsConst(m_rule->artifacts)) {
-            Artifact * const outputArtifact
-                    = createOutputArtifactFromRuleArtifact(ruleArtifact, inputArtifacts,
-                                                           &outputFilePaths);
-            if (!outputArtifact)
+        for (const RuleArtifactConstPtr &ruleArtifact : m_rule->artifacts) {
+            const OutputArtifactInfo outputInfo = createOutputArtifactFromRuleArtifact(
+                        ruleArtifact, inputArtifacts, &outputFilePaths);
+            if (!outputInfo.artifact)
                 continue;
-            outputArtifacts.push_back(outputArtifact);
-            ruleArtifactArtifactMap.push_back({ ruleArtifact.get(), outputArtifact });
+            outputArtifacts.push_back(outputInfo.artifact);
+            ruleArtifactArtifactMap.push_back({ ruleArtifact.get(), outputInfo });
+        }
+        if (m_rule->artifacts.empty()) {
+            outputArtifacts.push_back(createOutputArtifactFromRuleArtifact(
+                                          nullptr, inputArtifacts, &outputFilePaths).artifact);
+        }
+    }
+
+    ArtifactSet newOutputs = ArtifactSet::fromList(outputArtifacts);
+    const ArtifactSet oldOutputs = collectOldOutputArtifacts(inputArtifacts);
+    handleRemovedRuleOutputs(m_completeInputSet, oldOutputs - newOutputs, m_removedArtifacts,
+                             m_logger);
+
+    // The inputs become children of the rule node. Generated artifacts in the same product
+    // already are children, because output artifacts become children of the producing
+    // rule node's parent rule node.
+    for (Artifact * const input : inputArtifacts) {
+        if (input->artifactType == Artifact::SourceFile || input->product != m_ruleNode->product
+                || input->producer()->rule()->collectedOutputFileTags().intersects(
+                    m_ruleNode->rule()->excludedInputs)) {
+            connect(m_ruleNode, input);
+        } else {
+            QBS_CHECK(m_ruleNode->children.contains(input));
         }
     }
 
@@ -209,7 +249,6 @@ void RulesApplicator::doApply(const ArtifactSet &inputArtifacts, QScriptValue &p
     for (Artifact * const outputArtifact : qAsConst(outputArtifacts)) {
         for (Artifact * const dependency : qAsConst(m_transformer->explicitlyDependsOn))
             connect(outputArtifact, dependency);
-        outputArtifact->product->unregisterArtifactWithChangedInputs(outputArtifact);
     }
 
     if (inputArtifacts != m_transformer->inputs)
@@ -226,7 +265,8 @@ void RulesApplicator::doApply(const ArtifactSet &inputArtifacts, QScriptValue &p
             continue;
 
         // expose attributes of this artifact
-        Artifact *outputArtifact = it->second;
+        const OutputArtifactInfo outputInfo = it->second;
+        Artifact *outputArtifact = outputInfo.artifact;
         outputArtifact->properties = outputArtifact->properties->clone();
 
         scope().setProperty(StringConstants::fileNameProperty(),
@@ -238,14 +278,20 @@ void RulesApplicator::doApply(const ArtifactSet &inputArtifacts, QScriptValue &p
         for (const auto &binding : ra->bindings) {
             scriptValue = engine()->evaluate(binding.code);
             if (Q_UNLIKELY(engine()->hasErrorOrException(scriptValue))) {
-                QString msg = QLatin1String("evaluating rule binding '%1': %2");
+                QString msg = QStringLiteral("evaluating rule binding '%1': %2");
                 throw ErrorInfo(msg.arg(binding.name.join(QLatin1Char('.')),
                                         engine()->lastErrorString(scriptValue)),
                                 engine()->lastErrorLocation(scriptValue, binding.location));
             }
-            setConfigProperty(artifactModulesCfg, binding.name, scriptValue.toVariant());
+            const QVariant value = scriptValue.toVariant();
+            setConfigProperty(artifactModulesCfg, binding.name, value);
+            outputArtifact->pureProperties.push_back(std::make_pair(binding.name, value));
         }
         outputArtifact->properties->setValue(artifactModulesCfg);
+        if (!outputInfo.newlyCreated && (outputArtifact->fileTags() != outputInfo.oldFileTags
+                || outputArtifact->properties->value() != outputInfo.oldProperties)) {
+            invalidateArtifactAsRuleInputIfNecessary(outputArtifact);
+        }
     }
     if (!ruleArtifactArtifactMap.empty())
         engine()->setGlobalObject(prepareScriptContext.prototype());
@@ -256,6 +302,18 @@ void RulesApplicator::doApply(const ArtifactSet &inputArtifacts, QScriptValue &p
     if (Q_UNLIKELY(m_transformer->commands.empty()))
         throw ErrorInfo(Tr::tr("There is a rule without commands: %1.")
                         .arg(m_rule->toString()), m_rule->prepareScript.location());
+    if (!m_oldTransformer || m_oldTransformer->outputs != m_transformer->outputs
+            || m_oldTransformer->inputs != m_transformer->inputs
+            || m_oldTransformer->explicitlyDependsOn != m_transformer->explicitlyDependsOn
+            || m_oldTransformer->commands != m_transformer->commands
+            || commandsNeedRerun(m_transformer.get(), m_product.get(), m_productsByName,
+                                 m_projectsByName)) {
+        for (Artifact * const output : outputArtifacts) {
+            output->clearTimestamp();
+            m_invalidatedArtifacts += output;
+        }
+    }
+    m_transformer->commandsNeedChangeTracking = false;
 }
 
 ArtifactSet RulesApplicator::collectOldOutputArtifacts(const ArtifactSet &inputArtifacts) const
@@ -271,49 +329,108 @@ ArtifactSet RulesApplicator::collectOldOutputArtifacts(const ArtifactSet &inputA
     return result;
 }
 
-ArtifactSet RulesApplicator::collectExplicitlyDependsOn()
+ArtifactSet RulesApplicator::collectAdditionalInputs(const FileTags &tags, const Rule *rule,
+                                                     const ResolvedProduct *product,
+                                                     InputsSources inputsSources)
 {
     ArtifactSet artifacts;
-    for (const FileTag &fileTag : qAsConst(m_rule->explicitlyDependsOn)) {
-        for (Artifact *dependency : m_product->lookupArtifactsByFileTag(fileTag))
-            artifacts << dependency;
-        for (const ResolvedProductConstPtr &depProduct : qAsConst(m_product->dependencies)) {
-            for (Artifact * const ta : depProduct->targetArtifacts()) {
-                if (ta->fileTags().contains(fileTag))
-                    artifacts << ta;
+    for (const FileTag &fileTag : tags) {
+        for (Artifact *dependency : product->lookupArtifactsByFileTag(fileTag)) {
+            // Skip excluded inputs.
+            if (dependency->fileTags().intersects(rule->excludedInputs))
+                continue;
+
+            // Two cases are considered:
+            // 1) An artifact is considered a dependency when it's part of the current product.
+            // 2) An artifact marked with filesAreTargets: true inside a Group inside of a
+            // Module also ends up in the results returned by product->lookupArtifactsByFileTag,
+            // so it should be considered conceptually as a "dependent product artifact".
+            if ((inputsSources.testFlag(CurrentProduct) && !dependency->isTargetOfModule())
+                 || (inputsSources.testFlag(Dependencies) && dependency->isTargetOfModule())) {
+                artifacts << dependency;
+            }
+        }
+
+        if (inputsSources.testFlag(Dependencies)) {
+            for (const ResolvedProductConstPtr &depProduct : product->dependencies) {
+                for (Artifact * const ta : depProduct->targetArtifacts()) {
+                    if (ta->fileTags().contains(fileTag)
+                            && !ta->fileTags().intersects(rule->excludedInputs)) {
+                        artifacts << ta;
+                    }
+                }
             }
         }
     }
     return artifacts;
 }
 
-Artifact *RulesApplicator::createOutputArtifactFromRuleArtifact(
+ArtifactSet RulesApplicator::collectExplicitlyDependsOn(const Rule *rule,
+                                                        const ResolvedProduct *product)
+{
+   ArtifactSet first = collectAdditionalInputs(
+               rule->explicitlyDependsOn, rule, product, CurrentProduct);
+   ArtifactSet second = collectAdditionalInputs(
+               rule->explicitlyDependsOnFromDependencies, rule, product, Dependencies);
+   return first.unite(second);
+}
+
+RulesApplicator::OutputArtifactInfo RulesApplicator::createOutputArtifactFromRuleArtifact(
         const RuleArtifactConstPtr &ruleArtifact, const ArtifactSet &inputArtifacts,
         Set<QString> *outputFilePaths)
 {
-    QScriptValue scriptValue = engine()->evaluate(ruleArtifact->filePath,
-                                                  ruleArtifact->filePathLocation.filePath(),
-                                                  ruleArtifact->filePathLocation.line());
-    if (Q_UNLIKELY(engine()->hasErrorOrException(scriptValue)))
-        throw engine()->lastError(scriptValue, ruleArtifact->filePathLocation);
-    QString outputPath = FileInfo::resolvePath(m_product->buildDirectory(), scriptValue.toString());
+    QString outputPath;
+    FileTags fileTags;
+    bool alwaysUpdated;
+    if (ruleArtifact) {
+        QScriptValue scriptValue = engine()->evaluate(ruleArtifact->filePath,
+                                                      ruleArtifact->filePathLocation.filePath(),
+                                                      ruleArtifact->filePathLocation.line());
+        if (Q_UNLIKELY(engine()->hasErrorOrException(scriptValue)))
+            throw engine()->lastError(scriptValue, ruleArtifact->filePathLocation);
+        outputPath = scriptValue.toString();
+        fileTags = ruleArtifact->fileTags;
+        alwaysUpdated = ruleArtifact->alwaysUpdated;
+    } else {
+        outputPath = QStringLiteral("__dummyoutput__");
+        QByteArray hashInput = m_rule->toString().toLatin1();
+        for (const Artifact * const input : inputArtifacts)
+            hashInput += input->filePath().toLatin1();
+        outputPath += QLatin1String(QCryptographicHash::hash(hashInput, QCryptographicHash::Sha1)
+                                    .toHex().left(16));
+        fileTags = m_rule->outputFileTags;
+        alwaysUpdated = false;
+    }
+    outputPath = FileInfo::resolvePath(m_product->buildDirectory(), outputPath);
     if (Q_UNLIKELY(!outputFilePaths->insert(outputPath).second)) {
         throw ErrorInfo(Tr::tr("Rule %1 already created '%2'.")
                         .arg(m_rule->toString(), outputPath));
     }
-    return createOutputArtifact(outputPath, ruleArtifact->fileTags, ruleArtifact->alwaysUpdated,
-                                inputArtifacts);
+    return createOutputArtifact(outputPath, fileTags, alwaysUpdated, inputArtifacts);
 }
 
-Artifact *RulesApplicator::createOutputArtifact(const QString &filePath, const FileTags &fileTags,
-        bool alwaysUpdated, const ArtifactSet &inputArtifacts)
+RulesApplicator::OutputArtifactInfo RulesApplicator::createOutputArtifact(const QString &filePath,
+        const FileTags &fileTags, bool alwaysUpdated, const ArtifactSet &inputArtifacts)
 {
     QString outputPath = filePath;
     // don't let the output artifact "escape" its build dir
     outputPath.replace(StringConstants::dotDot(), QStringLiteral("dotdot"));
     outputPath = resolveOutPath(outputPath);
 
-    Artifact *outputArtifact = lookupArtifact(m_product, outputPath);
+    if (m_rule->isDynamic()) {
+        const Set<FileTag> undeclaredTags = fileTags - m_rule->collectedOutputFileTags();
+        if (!undeclaredTags.empty()) {
+            throw ErrorInfo(Tr::tr("Artifact '%1' has undeclared file tags [\"%2\"].")
+                            .arg(outputPath, undeclaredTags.toStringList()
+                                 .join(QLatin1String("\",\""))),
+                            m_rule->prepareScript.location());
+        }
+    }
+
+    OutputArtifactInfo outputInfo;
+    Artifact *& outputArtifact = outputInfo.artifact;
+    outputArtifact = lookupArtifact(m_product, outputPath);
+    outputInfo.newlyCreated = !outputArtifact;
     if (outputArtifact) {
         const Transformer * const transformer = outputArtifact->transformer.get();
         if (transformer && transformer->rule != m_rule) {
@@ -326,13 +443,13 @@ Artifact *RulesApplicator::createOutputArtifact(const QString &filePath, const F
                + QLatin1String("] -> [") + outputArtifact->fileTags().toStringList()
                     .join(QLatin1String(", ")) + QLatin1Char(']');
 
-            e += QString::fromLatin1("  while trying to apply:   %1:%2:%3  %4\n")
+            e += QStringLiteral("  while trying to apply:   %1:%2:%3  %4\n")
                 .arg(m_rule->prepareScript.location().filePath())
                 .arg(m_rule->prepareScript.location().line())
                 .arg(m_rule->prepareScript.location().column())
                 .arg(str);
 
-            e += QString::fromLatin1("  was already defined in:  %1:%2:%3  %4\n")
+            e += QStringLiteral("  was already defined in:  %1:%2:%3  %4\n")
                 .arg(transformer->rule->prepareScript.location().filePath())
                 .arg(transformer->rule->prepareScript.location().line())
                 .arg(transformer->rule->prepareScript.location().column())
@@ -352,10 +469,10 @@ Artifact *RulesApplicator::createOutputArtifact(const QString &filePath, const F
                               (*inputArtifacts.cbegin())->filePath()));
             throw error;
         }
-        if (m_rule->declaresInputs() && m_rule->requiresInputs)
-            outputArtifact->clearTimestamp();
-        m_invalidatedArtifacts += outputArtifact;
         m_transformer->rescueChangeTrackingData(outputArtifact->transformer);
+        m_oldTransformer = outputArtifact->transformer;
+        outputInfo.oldFileTags = outputArtifact->fileTags();
+        outputInfo.oldProperties = outputArtifact->properties->value();
     } else {
         std::unique_ptr<Artifact> newArtifact(new Artifact);
         newArtifact->artifactType = Artifact::Generated;
@@ -363,25 +480,17 @@ Artifact *RulesApplicator::createOutputArtifact(const QString &filePath, const F
         insertArtifact(m_product, newArtifact.get());
         m_createdArtifacts += newArtifact.get();
         outputArtifact = newArtifact.release();
+        qCDebug(lcExec).noquote() << "rule created" << outputArtifact->toString();
+        connect(outputArtifact, m_ruleNode);
     }
 
     outputArtifact->alwaysUpdated = alwaysUpdated;
-    outputArtifact->properties = m_product->moduleProperties;
-
-    FileTags outputArtifactFileTags = fileTags.empty()
-            ? m_product->fileTagsForFileName(outputArtifact->fileName()) : fileTags;
-    for (const ArtifactPropertiesConstPtr &props : qAsConst(m_product->artifactProperties)) {
-        if (outputArtifactFileTags.intersects(props->fileTagsFilter())) {
-            outputArtifact->properties = props->propertyMap();
-            outputArtifactFileTags += props->extraFileTags();
-            break;
-        }
+    outputArtifact->pureFileTags = fileTags;
+    provideFullFileTagsAndProperties(outputArtifact);
+    if (outputInfo.newlyCreated || outputInfo.oldFileTags != outputArtifact->fileTags()) {
+        for (RuleNode * const parentRule : filterByType<RuleNode>(m_ruleNode->parents))
+            connect(parentRule, outputArtifact);
     }
-    outputArtifact->setFileTags(outputArtifactFileTags);
-
-    // Let a positive value of qbs.install imply the file tag "installable".
-    if (outputArtifact->properties->qbsPropertyValue(StringConstants::installProperty()).toBool())
-        outputArtifact->addFileTag("installable");
 
     for (Artifact * const inputArtifact : inputArtifacts) {
         QBS_CHECK(outputArtifact != inputArtifact);
@@ -392,7 +501,7 @@ Artifact *RulesApplicator::createOutputArtifact(const QString &filePath, const F
     m_transformer->outputs.insert(outputArtifact);
     QBS_CHECK(m_rule->multiplex || m_transformer->inputs.size() == 1);
 
-    return outputArtifact;
+    return outputInfo;
 }
 
 class RuleOutputArtifactsException : public ErrorInfo
@@ -409,7 +518,7 @@ QList<Artifact *> RulesApplicator::runOutputArtifactsScript(const ArtifactSet &i
                                           m_rule->outputArtifactsScript.location().filePath(),
                                           m_rule->outputArtifactsScript.location().line());
     if (!fun.isFunction())
-        throw ErrorInfo(QLatin1String("Function expected."),
+        throw ErrorInfo(QStringLiteral("Function expected."),
                         m_rule->outputArtifactsScript.location());
     QScriptValue res = fun.call(QScriptValue(), args);
     engine()->releaseResourcesOfScriptObjects();
@@ -493,8 +602,11 @@ public:
 
         outputArtifact->properties = outputArtifact->properties->clone();
         QVariantMap artifactCfg = outputArtifact->properties->value();
-        for (const auto &e : m_propertyValues)
-            setConfigProperty(artifactCfg, {e.module, e.name}, e.value);
+        for (const auto &e : m_propertyValues) {
+            const QStringList key{e.module, e.name};
+            setConfigProperty(artifactCfg, key, e.value);
+            outputArtifact->pureProperties.push_back(std::make_pair(key, e.value));
+        }
         outputArtifact->properties->setValue(artifactCfg);
     }
 };
@@ -518,8 +630,9 @@ Artifact *RulesApplicator::createOutputArtifactFromScriptValue(const QScriptValu
     const QVariant alwaysUpdatedVar
             = obj.property(StringConstants::alwaysUpdatedProperty()).toVariant();
     const bool alwaysUpdated = alwaysUpdatedVar.isValid() ? alwaysUpdatedVar.toBool() : true;
-    Artifact *output = createOutputArtifact(filePath, fileTags, alwaysUpdated, inputArtifacts);
-    if (output->fileTags().empty()) {
+    OutputArtifactInfo outputInfo = createOutputArtifact(filePath, fileTags, alwaysUpdated,
+                                                         inputArtifacts);
+    if (outputInfo.artifact->fileTags().empty()) {
         // Check the file tags after file taggers were run.
         throw RuleOutputArtifactsException(
                 Tr::tr("Property fileTags for artifact '%1' must be a non-empty string list. "
@@ -531,10 +644,14 @@ Artifact *RulesApplicator::createOutputArtifactFromScriptValue(const QScriptValu
                 .toVariant().toStringList());
     for (const FileTag &tag : explicitlyDependsOn) {
         for (Artifact * const dependency : m_product->lookupArtifactsByFileTag(tag))
-            connect(output, dependency);
+            connect(outputInfo.artifact, dependency);
     }
-    ArtifactBindingsExtractor().apply(output, obj);
-    return output;
+    ArtifactBindingsExtractor().apply(outputInfo.artifact, obj);
+    if (!outputInfo.newlyCreated && (outputInfo.artifact->fileTags() != outputInfo.oldFileTags
+            || outputInfo.artifact->properties->value() != outputInfo.oldProperties)) {
+        invalidateArtifactAsRuleInputIfNecessary(outputInfo.artifact);
+    }
+    return outputInfo.artifact;
 }
 
 QString RulesApplicator::resolveOutPath(const QString &path) const
