@@ -37,17 +37,18 @@
 **
 ****************************************************************************/
 
-#include "fileinfoextension.h"
-
 #include <language/scriptengine.h>
 #include <logging/translator.h>
 #include <tools/fileinfo.h>
+#include <tools/stringconstants.h>
 
-#include <QDir>
-#include <QFileInfo>
-#include <QRegularExpression>
-#include <QScriptable>
-#include <QScriptEngine>
+#include <QtCore/qdir.h>
+#include <QtCore/qfileinfo.h>
+
+#include <QtScript/qscriptable.h>
+#include <QtScript/qscriptengine.h>
+
+#include <regex>
 
 namespace qbs {
 namespace Internal {
@@ -56,13 +57,14 @@ class FileInfoExtension : public QObject, QScriptable
 {
     Q_OBJECT
 public:
-    friend void initializeJsExtensionFileInfo(QScriptValue extensionObject);
-
-private:
     static QScriptValue js_ctor(QScriptContext *context, QScriptEngine *engine);
     static QScriptValue js_path(QScriptContext *context, QScriptEngine *engine);
     static QScriptValue js_fileName(QScriptContext *context, QScriptEngine *engine);
     static QScriptValue js_baseName(QScriptContext *context, QScriptEngine *engine);
+    static QScriptValue js_suffix(QScriptContext *context, QScriptEngine *engine);
+    static QScriptValue js_completeSuffix(QScriptContext *context, QScriptEngine *engine);
+    static QScriptValue js_canonicalPath(QScriptContext *context, QScriptEngine *engine);
+    static QScriptValue js_cleanPath(QScriptContext *context, QScriptEngine *engine);
     static QScriptValue js_completeBaseName(QScriptContext *context, QScriptEngine *engine);
     static QScriptValue js_relativePath(QScriptContext *context, QScriptEngine *engine);
     static QScriptValue js_resolvePath(QScriptContext *context, QScriptEngine *engine);
@@ -74,38 +76,6 @@ private:
     static QScriptValue js_joinPaths(QScriptContext *context, QScriptEngine *engine);
 };
 
-void initializeJsExtensionFileInfo(QScriptValue extensionObject)
-{
-    QScriptEngine *engine = extensionObject.engine();
-    QScriptValue fileInfoObj = engine->newQMetaObject(&FileInfoExtension::staticMetaObject,
-                                                  engine->newFunction(&FileInfoExtension::js_ctor));
-    fileInfoObj.setProperty(QLatin1String("path"),
-                            engine->newFunction(FileInfoExtension::js_path));
-    fileInfoObj.setProperty(QLatin1String("fileName"),
-                            engine->newFunction(FileInfoExtension::js_fileName));
-    fileInfoObj.setProperty(QLatin1String("baseName"),
-                            engine->newFunction(FileInfoExtension::js_baseName));
-    fileInfoObj.setProperty(QLatin1String("completeBaseName"),
-                            engine->newFunction(FileInfoExtension::js_completeBaseName));
-    fileInfoObj.setProperty(QLatin1String("relativePath"),
-                            engine->newFunction(FileInfoExtension::js_relativePath));
-    fileInfoObj.setProperty(QLatin1String("resolvePath"),
-                            engine->newFunction(FileInfoExtension::js_resolvePath));
-    fileInfoObj.setProperty(QLatin1String("isAbsolutePath"),
-                            engine->newFunction(FileInfoExtension::js_isAbsolutePath));
-    fileInfoObj.setProperty(QLatin1String("toWindowsSeparators"),
-                            engine->newFunction(FileInfoExtension::js_toWindowsSeparators));
-    fileInfoObj.setProperty(QLatin1String("fromWindowsSeparators"),
-                            engine->newFunction(FileInfoExtension::js_fromWindowsSeparators));
-    fileInfoObj.setProperty(QLatin1String("toNativeSeparators"),
-                            engine->newFunction(FileInfoExtension::js_toWindowsSeparators));
-    fileInfoObj.setProperty(QLatin1String("fromNativeSeparators"),
-                            engine->newFunction(FileInfoExtension::js_fromWindowsSeparators));
-    fileInfoObj.setProperty(QLatin1String("joinPaths"),
-                            engine->newFunction(FileInfoExtension::js_joinPaths));
-    extensionObject.setProperty(QLatin1String("FileInfo"), fileInfoObj);
-}
-
 QScriptValue FileInfoExtension::js_ctor(QScriptContext *context, QScriptEngine *engine)
 {
     Q_UNUSED(engine);
@@ -147,6 +117,46 @@ QScriptValue FileInfoExtension::js_baseName(QScriptContext *context, QScriptEngi
     return FileInfo::baseName(context->argument(0).toString());
 }
 
+QScriptValue FileInfoExtension::js_suffix(QScriptContext *context, QScriptEngine *engine)
+{
+    Q_UNUSED(engine);
+    if (Q_UNLIKELY(context->argumentCount() < 1)) {
+        return context->throwError(QScriptContext::SyntaxError,
+                                   Tr::tr("suffix expects 1 argument"));
+    }
+    return FileInfo::suffix(context->argument(0).toString());
+}
+
+QScriptValue FileInfoExtension::js_completeSuffix(QScriptContext *context, QScriptEngine *engine)
+{
+    Q_UNUSED(engine);
+    if (Q_UNLIKELY(context->argumentCount() < 1)) {
+        return context->throwError(QScriptContext::SyntaxError,
+                                   Tr::tr("completeSuffix expects 1 argument"));
+    }
+    return FileInfo::completeSuffix(context->argument(0).toString());
+}
+
+QScriptValue FileInfoExtension::js_canonicalPath(QScriptContext *context, QScriptEngine *engine)
+{
+    Q_UNUSED(engine);
+    if (Q_UNLIKELY(context->argumentCount() < 1)) {
+        return context->throwError(QScriptContext::SyntaxError,
+                                   Tr::tr("canonicalPath expects 1 argument"));
+    }
+    return QFileInfo(context->argument(0).toString()).canonicalFilePath();
+}
+
+QScriptValue FileInfoExtension::js_cleanPath(QScriptContext *context, QScriptEngine *engine)
+{
+    Q_UNUSED(engine);
+    if (Q_UNLIKELY(context->argumentCount() < 1)) {
+        return context->throwError(QScriptContext::SyntaxError,
+                                   Tr::tr("cleanPath expects 1 argument"));
+    }
+    return QDir::cleanPath(context->argument(0).toString());
+}
+
 QScriptValue FileInfoExtension::js_completeBaseName(QScriptContext *context, QScriptEngine *engine)
 {
     Q_UNUSED(engine);
@@ -164,9 +174,19 @@ QScriptValue FileInfoExtension::js_relativePath(QScriptContext *context, QScript
         return context->throwError(QScriptContext::SyntaxError,
                                    Tr::tr("relativePath expects 2 arguments"));
     }
-    const QString base = context->argument(0).toString();
-    const QString rel = context->argument(1).toString();
-    return QDir(base).relativeFilePath(rel);
+    const QString baseDir = context->argument(0).toString();
+    const QString filePath = context->argument(1).toString();
+    if (!FileInfo::isAbsolute(baseDir)) {
+        return context->throwError(QScriptContext::SyntaxError,
+                                   Tr::tr("FileInfo.relativePath() expects an absolute path as "
+                                          "its first argument, but it is '%1'.").arg(baseDir));
+    }
+    if (!FileInfo::isAbsolute(filePath)) {
+        return context->throwError(QScriptContext::SyntaxError,
+                                   Tr::tr("FileInfo.relativePath() expects an absolute path as "
+                                          "its second argument, but it is '%1'.").arg(filePath));
+    }
+    return QDir(baseDir).relativeFilePath(filePath);
 }
 
 QScriptValue FileInfoExtension::js_resolvePath(QScriptContext *context, QScriptEngine *engine)
@@ -245,16 +265,58 @@ QScriptValue FileInfoExtension::js_joinPaths(QScriptContext *context, QScriptEng
         if (!value.isUndefined() && !value.isNull()) {
             const QString arg = value.toString();
             if (!arg.isEmpty())
-                paths.append(arg);
+                paths.push_back(arg);
         }
     }
-    return paths.join(QLatin1Char('/')).replace(QRegularExpression(QLatin1String("/{2,}")),
-                                                QLatin1String("/"));
+    return engine->toScriptValue(QString::fromStdString(
+                                     std::regex_replace(paths.join(QLatin1Char('/')).toStdString(),
+                                                        std::regex("/{2,}"), std::string("/"))));
 }
 
 } // namespace Internal
 } // namespace qbs
 
+void initializeJsExtensionFileInfo(QScriptValue extensionObject)
+{
+    using namespace qbs::Internal;
+    QScriptEngine *engine = extensionObject.engine();
+    QScriptValue fileInfoObj = engine->newQMetaObject(&FileInfoExtension::staticMetaObject,
+                                                  engine->newFunction(&FileInfoExtension::js_ctor));
+    fileInfoObj.setProperty(StringConstants::fileInfoPath(),
+                            engine->newFunction(FileInfoExtension::js_path));
+    fileInfoObj.setProperty(StringConstants::fileInfoFileName(),
+                            engine->newFunction(FileInfoExtension::js_fileName));
+    fileInfoObj.setProperty(StringConstants::baseNameProperty(),
+                            engine->newFunction(FileInfoExtension::js_baseName));
+    fileInfoObj.setProperty(QStringLiteral("suffix"),
+                            engine->newFunction(FileInfoExtension::js_suffix));
+    fileInfoObj.setProperty(QStringLiteral("completeSuffix"),
+                            engine->newFunction(FileInfoExtension::js_completeSuffix));
+    fileInfoObj.setProperty(QStringLiteral("canonicalPath"),
+                            engine->newFunction(FileInfoExtension::js_canonicalPath));
+    fileInfoObj.setProperty(QStringLiteral("cleanPath"),
+                            engine->newFunction(FileInfoExtension::js_cleanPath));
+    fileInfoObj.setProperty(StringConstants::completeBaseNameProperty(),
+                            engine->newFunction(FileInfoExtension::js_completeBaseName));
+    fileInfoObj.setProperty(QStringLiteral("relativePath"),
+                            engine->newFunction(FileInfoExtension::js_relativePath));
+    fileInfoObj.setProperty(QStringLiteral("resolvePath"),
+                            engine->newFunction(FileInfoExtension::js_resolvePath));
+    fileInfoObj.setProperty(QStringLiteral("isAbsolutePath"),
+                            engine->newFunction(FileInfoExtension::js_isAbsolutePath));
+    fileInfoObj.setProperty(QStringLiteral("toWindowsSeparators"),
+                            engine->newFunction(FileInfoExtension::js_toWindowsSeparators));
+    fileInfoObj.setProperty(QStringLiteral("fromWindowsSeparators"),
+                            engine->newFunction(FileInfoExtension::js_fromWindowsSeparators));
+    fileInfoObj.setProperty(QStringLiteral("toNativeSeparators"),
+                            engine->newFunction(FileInfoExtension::js_toNativeSeparators));
+    fileInfoObj.setProperty(QStringLiteral("fromNativeSeparators"),
+                            engine->newFunction(FileInfoExtension::js_fromNativeSeparators));
+    fileInfoObj.setProperty(QStringLiteral("joinPaths"),
+                            engine->newFunction(FileInfoExtension::js_joinPaths));
+    extensionObject.setProperty(QStringLiteral("FileInfo"), fileInfoObj);
+}
+
 Q_DECLARE_METATYPE(qbs::Internal::FileInfoExtension *)
 
 #include "fileinfoextension.moc"
