@@ -44,6 +44,7 @@
 #include <logging/translator.h>
 #include <tools/error.h>
 #include <tools/qbsassert.h>
+#include <tools/stringconstants.h>
 
 namespace qbs {
 namespace Internal {
@@ -64,19 +65,23 @@ void ASTPropertiesItemHandler::setupAlternatives()
     auto it = m_parentItem->m_children.begin();
     while (it != m_parentItem->m_children.end()) {
         Item * const child = *it;
+        bool remove = false;
         if (child->type() == ItemType::Properties) {
             handlePropertiesBlock(child);
-            it = m_parentItem->m_children.erase(it);
-        } else {
-            ++it;
+            remove = m_parentItem->type() != ItemType::Export;
         }
+        if (remove)
+            it = m_parentItem->m_children.erase(it);
+        else
+            ++it;
     }
 }
 
 class PropertiesBlockConverter
 {
 public:
-    PropertiesBlockConverter(const QString &condition, const QString &overrideListProperties,
+    PropertiesBlockConverter(const JSSourceValue::AltProperty &condition,
+                             const JSSourceValue::AltProperty &overrideListProperties,
                              Item *propertiesBlockContainer, const Item *propertiesBlock)
         : m_propertiesBlockContainer(propertiesBlockContainer)
         , m_propertiesBlock(propertiesBlock)
@@ -100,8 +105,8 @@ private:
         for (auto it = inner->properties().constBegin();
                 it != inner->properties().constEnd(); ++it) {
             if (inner == m_propertiesBlock
-                    && (it.key() == QLatin1String("condition")
-                        || it.key() == QLatin1String("overrideListProperties"))) {
+                    && (it.key() == StringConstants::conditionProperty()
+                        || it.key() == StringConstants::overrideListPropertiesProperty())) {
                 continue;
             }
             if (it.value()->type() == Value::ItemValueType) {
@@ -130,13 +135,11 @@ private:
     void doApply(const QString &propertyName, Item *item, JSSourceValuePtr value,
                const JSSourceValuePtr &conditionalValue)
     {
-        QBS_ASSERT(!value || value->file() == conditionalValue->file(), return);
         if (!value) {
             value = JSSourceValue::create(true);
             value->setFile(conditionalValue->file());
             item->setProperty(propertyName, value);
-            static const QString baseKeyword = QLatin1String("base");
-            value->setSourceCode(QStringRef(&baseKeyword));
+            value->setSourceCode(QStringRef(&StringConstants::baseVar()));
             value->setSourceUsesBaseFlag();
         }
         m_alternative.value = conditionalValue;
@@ -144,21 +147,22 @@ private:
     }
 };
 
-static QString getPropertyString(const Item *propertiesItem, const QString &name)
+static JSSourceValue::AltProperty getPropertyData(const Item *propertiesItem, const QString &name)
 {
     const ValuePtr value = propertiesItem->property(name);
     if (!value) {
-        if (name == QLatin1String("condition")) {
+        if (name == StringConstants::conditionProperty()) {
             throw ErrorInfo(Tr::tr("Properties.condition must be provided."),
                             propertiesItem->location());
         }
-        return QLatin1String("false");
+        return JSSourceValue::AltProperty(StringConstants::falseValue(),
+                                          propertiesItem->location());
     }
     if (Q_UNLIKELY(value->type() != Value::JSSourceValueType)) {
         throw ErrorInfo(Tr::tr("Properties.%1 must be a value binding.").arg(name),
                     propertiesItem->location());
     }
-    if (name == QLatin1String("overrideListProperties")) {
+    if (name == StringConstants::overrideListPropertiesProperty()) {
         const Item *parent = propertiesItem->parent();
         while (parent) {
             if (parent->type() == ItemType::Product)
@@ -172,14 +176,14 @@ static QString getPropertyString(const Item *propertiesItem, const QString &name
 
     }
     const JSSourceValuePtr srcval = std::static_pointer_cast<JSSourceValue>(value);
-    return srcval->sourceCodeForEvaluation();
+    return JSSourceValue::AltProperty(srcval->sourceCodeForEvaluation(), srcval->location());
 }
 
 void ASTPropertiesItemHandler::handlePropertiesBlock(const Item *propertiesItem)
 {
-    const QString condition = getPropertyString(propertiesItem, QLatin1String("condition"));
-    const QString overrideListProperties = getPropertyString(propertiesItem,
-            QLatin1String("overrideListProperties"));
+    const auto condition = getPropertyData(propertiesItem, StringConstants::conditionProperty());
+    const auto overrideListProperties = getPropertyData(propertiesItem,
+            StringConstants::overrideListPropertiesProperty());
     PropertiesBlockConverter(condition, overrideListProperties, m_parentItem,
                              propertiesItem).apply();
 }
