@@ -131,7 +131,8 @@ private:
                 = itemOfProperty->propertyDeclaration(propertyName->toString()).type();
         const bool isArray = type == PropertyDeclaration::StringList
                 || type == PropertyDeclaration::PathList
-                || type == PropertyDeclaration::Variant;
+                || type == PropertyDeclaration::Variant // TODO: Why?
+                || type == PropertyDeclaration::VariantList;
         QScriptValue valueToSet = scriptValue;
         if (isArray) {
             if (!valueToSet.isValid() || valueToSet.isUndefined())
@@ -176,6 +177,25 @@ private:
             QScriptValue originalValue;
             if (data->item->propertyDeclaration(propertyName->toString()).isScalar()) {
                 const Item *item = itemOfProperty;
+                if (item->type() == ItemType::Module || item->type() == ItemType::Export) {
+                    const QString errorMessage = Tr::tr("The special value 'original' cannot "
+                            "be used on the right-hand side of a property declaration.");
+                    extraScope = engine->currentContext()->throwError(errorMessage);
+                    result.second = false;
+                    return result;
+                }
+
+                // TODO: Provide a dedicated item type for not-yet-instantiated things that
+                //       look like module instances in the AST visitor.
+                if (item->type() == ItemType::ModuleInstance
+                        && !item->hasProperty(StringConstants::presentProperty())) {
+                    const QString errorMessage = Tr::tr("Trying to assign property '%1' "
+                            "on something that is not a module.").arg(propertyName->toString());
+                    extraScope = engine->currentContext()->throwError(errorMessage);
+                    result.second = false;
+                    return result;
+                }
+
                 while (item->type() == ItemType::ModuleInstance)
                     item = item->prototype();
                 if (item->type() != ItemType::Module && item->type() != ItemType::Export) {
@@ -356,12 +376,12 @@ QScriptClass::QueryFlags EvaluatorScriptClass::queryProperty(const QScriptValue
 
     // We assume that it's safe to save the result of the query in a member of the scriptclass.
     // It must be cleared in the property method before doing any further lookup.
-    QBS_ASSERT(m_queryResult.isNull(), return QueryFlags());
+    QBS_ASSERT(m_queryResult.isNull(), return {});
 
     if (debugProperties)
         qDebug() << "[SC] queryProperty " << object.objectId() << " " << name;
 
-    EvaluationData *const data = attachedPointer<EvaluationData>(object);
+    auto const data = attachedPointer<EvaluationData>(object);
     const QString nameString = name.toString();
     if (nameString == QStringLiteral("parent")) {
         *id = QPTParentProperty;
@@ -373,7 +393,7 @@ QScriptClass::QueryFlags EvaluatorScriptClass::queryProperty(const QScriptValue
     if (!data) {
         if (debugProperties)
             qDebug() << "[SC] queryProperty: no data attached";
-        return QScriptClass::QueryFlags();
+        return {};
     }
 
     return queryItemProperty(data, nameString);
@@ -407,13 +427,13 @@ QScriptClass::QueryFlags EvaluatorScriptClass::queryItemProperty(const Evaluatio
 
     if (debugProperties)
         qDebug() << "[SC] queryProperty: no such property";
-    return QScriptClass::QueryFlags();
+    return {};
 }
 
 QString EvaluatorScriptClass::resultToString(const QScriptValue &scriptValue)
 {
     return (scriptValue.isObject()
-        ? QLatin1String("[Object: ")
+        ? QStringLiteral("[Object: ")
             + QString::number(scriptValue.objectId()) + QLatin1Char(']')
         : scriptValue.toVariant().toString());
 }
@@ -489,7 +509,7 @@ static void convertToPropertyType_impl(const QString &pathPropertiesBaseDir, con
     QString actualBaseDir;
     if (item && !pathPropertiesBaseDir.isEmpty()) {
         const VariantValueConstPtr itemSourceDir
-                = item->variantProperty(QLatin1String("sourceDirectory"));
+                = item->variantProperty(QStringLiteral("sourceDirectory"));
         actualBaseDir = itemSourceDir ? itemSourceDir->value().toString() : pathPropertiesBaseDir;
     }
     switch (decl.type()) {
@@ -561,6 +581,13 @@ static void convertToPropertyType_impl(const QString &pathPropertiesBaseDir, con
         }
         break;
     }
+    case PropertyDeclaration::VariantList:
+        if (!v.isArray()) {
+            QScriptValue x = v.engine()->newArray(1);
+            x.setProperty(0, v);
+            v = x;
+        }
+        break;
     }
 }
 
@@ -594,7 +621,8 @@ public:
                     || itemOfProperty->type() == ItemType::Export)) {
             const VariantValueConstPtr varValue
                     = itemOfProperty->variantProperty(StringConstants::nameProperty());
-            QBS_ASSERT(varValue, return);
+            if (!varValue)
+                return;
             m_stackUpdate = true;
             const QualifiedId fullPropName
                     = QualifiedId::fromString(varValue->value().toString()) << name.toString();
@@ -624,7 +652,7 @@ QScriptValue EvaluatorScriptClass::property(const QScriptValue &object, const QS
     m_queryResult.foundInParent = false;
     m_queryResult.data = nullptr;
     m_queryResult.itemOfProperty = nullptr;
-    QBS_ASSERT(data, return QScriptValue());
+    QBS_ASSERT(data, {});
 
     const auto qpt = static_cast<QueryPropertyType>(id);
     if (qpt == QPTParentProperty) {
@@ -635,8 +663,8 @@ QScriptValue EvaluatorScriptClass::property(const QScriptValue &object, const QS
 
     ValuePtr value;
     m_queryResult.value.swap(value);
-    QBS_ASSERT(value, return QScriptValue());
-    QBS_ASSERT(m_queryResult.isNull(), return QScriptValue());
+    QBS_ASSERT(value, return {});
+    QBS_ASSERT(m_queryResult.isNull(), return {});
 
     if (debugProperties)
         qDebug() << "[SC] property " << name;
@@ -724,7 +752,7 @@ private:
 
 QScriptClassPropertyIterator *EvaluatorScriptClass::newIterator(const QScriptValue &object)
 {
-    EvaluationData *const data = attachedPointer<EvaluationData>(object);
+    auto const data = attachedPointer<EvaluationData>(object);
     return data ? new EvaluatorScriptClassPropertyIterator(object, data) : nullptr;
 }
 
