@@ -1,44 +1,75 @@
 /****************************************************************************
 **
-** Copyright (C) 2015 The Qt Company Ltd.
-** Contact: http://www.qt.io/licensing
+** Copyright (C) 2016 The Qt Company Ltd.
+** Contact: https://www.qt.io/licensing/
 **
 ** This file is part of Qbs.
 **
+** $QT_BEGIN_LICENSE:LGPL$
 ** Commercial License Usage
 ** Licensees holding valid commercial Qt licenses may use this file in
 ** accordance with the commercial license agreement provided with the
 ** Software or, alternatively, in accordance with the terms contained in
-** a written agreement between you and The Qt Company. For licensing terms and
-** conditions see http://www.qt.io/terms-conditions. For further information
-** use the contact form at http://www.qt.io/contact-us.
+** a written agreement between you and The Qt Company. For licensing terms
+** and conditions see https://www.qt.io/terms-conditions. For further
+** information use the contact form at https://www.qt.io/contact-us.
 **
 ** GNU Lesser General Public License Usage
 ** Alternatively, this file may be used under the terms of the GNU Lesser
-** General Public License version 2.1 or version 3 as published by the Free
-** Software Foundation and appearing in the file LICENSE.LGPLv21 and
-** LICENSE.LGPLv3 included in the packaging of this file.  Please review the
-** following information to ensure the GNU Lesser General Public License
-** requirements will be met: https://www.gnu.org/licenses/lgpl.html and
-** http://www.gnu.org/licenses/old-licenses/lgpl-2.1.html.
+** General Public License version 3 as published by the Free Software
+** Foundation and appearing in the file LICENSE.LGPL3 included in the
+** packaging of this file. Please review the following information to
+** ensure the GNU Lesser General Public License version 3 requirements
+** will be met: https://www.gnu.org/licenses/lgpl-3.0.html.
 **
-** In addition, as a special exception, The Qt Company gives you certain additional
-** rights.  These rights are described in The Qt Company LGPL Exception
-** version 1.1, included in the file LGPL_EXCEPTION.txt in this package.
+** GNU General Public License Usage
+** Alternatively, this file may be used under the terms of the GNU
+** General Public License version 2.0 or (at your option) the GNU General
+** Public license version 3 or any later version approved by the KDE Free
+** Qt Foundation. The licenses are as published by the Free Software
+** Foundation and appearing in the file LICENSE.GPL2 and LICENSE.GPL3
+** included in the packaging of this file. Please review the following
+** information to ensure the GNU General Public License requirements will
+** be met: https://www.gnu.org/licenses/gpl-2.0.html and
+** https://www.gnu.org/licenses/gpl-3.0.html.
+**
+** $QT_END_LICENSE$
 **
 ****************************************************************************/
 
 #include "error.h"
 
-#include <QRegularExpression>
-#include <QSharedData>
-#include <QStringList>
+#include "persistence.h"
+#include "qttools.h"
+
+#include <QtCore/qshareddata.h>
+#include <QtCore/qstringlist.h>
+
+#include <algorithm>
+#include <functional>
+#include <regex>
 
 namespace qbs {
 
 class ErrorItem::ErrorItemPrivate : public QSharedData
 {
 public:
+    template<Internal::PersistentPool::OpType opType>
+    void serializationOp(Internal::PersistentPool &pool)
+    {
+        pool.serializationOp<opType>(description, codeLocation, isBacktraceItem);
+    }
+
+    void load(Internal::PersistentPool &pool)
+    {
+        serializationOp<Internal::PersistentPool::Load>(pool);
+    }
+
+    void store(Internal::PersistentPool &pool)
+    {
+        serializationOp<Internal::PersistentPool::Store>(pool);
+    }
+
     QString description;
     CodeLocation codeLocation;
     bool isBacktraceItem = false;
@@ -93,6 +124,16 @@ bool ErrorItem::isBacktraceItem() const
     return d->isBacktraceItem;
 }
 
+void ErrorItem::load(Internal::PersistentPool &pool)
+{
+    pool.load(*d);
+}
+
+void ErrorItem::store(Internal::PersistentPool &pool) const
+{
+    pool.store(*d);
+}
+
 /*!
  * \fn const QString &ErrorData::description() const
  * \brief A general description of the error.
@@ -121,6 +162,12 @@ class ErrorInfo::ErrorInfoPrivate : public QSharedData
 public:
     ErrorInfoPrivate() : internalError(false) { }
 
+    template<Internal::PersistentPool::OpType opType>
+    void completeSerializationOp(Internal::PersistentPool &pool)
+    {
+        pool.serializationOp<opType>(items, internalError);
+    }
+
     QList<ErrorItem> items;
     bool internalError;
 };
@@ -152,13 +199,15 @@ ErrorInfo::ErrorInfo(const QString &description, const QStringList &backtrace)
 {
     append(description);
     for (const QString &traceLine : backtrace) {
-        QRegularExpression regexp(
-                    QStringLiteral("^(?<message>.+) at (?<file>.+):(?<line>\\-?[0-9]+)$"));
-        QRegularExpressionMatch match = regexp.match(traceLine);
-        if (match.hasMatch()) {
-            const CodeLocation location(match.captured(QStringLiteral("file")),
-                                        match.captured(QStringLiteral("line")).toInt());
-            appendBacktrace(match.captured(QStringLiteral("message")), location);
+        static const std::regex regexp("^(.+) at (.+):(\\-?[0-9]+)$");
+        std::smatch match;
+        const std::string tl = traceLine.toStdString();
+        if (std::regex_match(tl, match, regexp)) {
+            const QString message = QString::fromStdString(match[1]),
+                             file = QString::fromStdString(match[2]),
+                             line = QString::fromStdString(match[3]);
+            const CodeLocation location(file, line.toInt());
+            appendBacktrace(message, location);
         }
     }
 }
@@ -176,12 +225,17 @@ ErrorInfo::~ErrorInfo()
 
 void ErrorInfo::appendBacktrace(const QString &description, const CodeLocation &location)
 {
-    d->items.append(ErrorItem(description, location, true));
+    d->items.push_back(ErrorItem(description, location, true));
+}
+
+void ErrorInfo::append(const ErrorItem &item)
+{
+    d->items.push_back(item);
 }
 
 void ErrorInfo::append(const QString &description, const CodeLocation &location)
 {
-    d->items.append(ErrorItem(description, location));
+    d->items.push_back(ErrorItem(description, location));
 }
 
 void ErrorInfo::prepend(const QString &description, const CodeLocation &location)
@@ -212,7 +266,7 @@ void ErrorInfo::clear()
 QString ErrorInfo::toString() const
 {
     QStringList lines;
-    foreach (const ErrorItem &e, d->items) {
+    for (const ErrorItem &e : qAsConst(d->items)) {
         if (e.isBacktraceItem()) {
             QString line;
             if (!e.description().isEmpty())
@@ -220,9 +274,9 @@ QString ErrorInfo::toString() const
             if (e.codeLocation().isValid())
                 line.append(QStringLiteral(" in %1").arg(e.codeLocation().toString()));
             if (!line.isEmpty())
-                lines.append(QStringLiteral("\t") + line);
+                lines.push_back(QLatin1Char('\t') + line);
         } else {
-            lines.append(e.toString());
+            lines.push_back(e.toString());
         }
     }
     return lines.join(QLatin1Char('\n'));
@@ -236,4 +290,27 @@ bool ErrorInfo::isInternalError() const
     return d->internalError;
 }
 
+bool ErrorInfo::hasLocation() const
+{
+    return std::any_of(d->items.cbegin(), d->items.cend(),
+                        [](const ErrorItem &ei) { return ei.codeLocation().isValid(); });
+}
+
+void ErrorInfo::load(Internal::PersistentPool &pool)
+{
+    pool.load(*d);
+}
+
+void ErrorInfo::store(Internal::PersistentPool &pool) const
+{
+    pool.store(*d);
+}
+
+void appendError(ErrorInfo &dst, const ErrorInfo &src)
+{
+    const QList<ErrorItem> &sourceItems = src.items();
+    for (const ErrorItem &item : sourceItems)
+        dst.append(item);
+}
+
 } // namespace qbs
