@@ -41,18 +41,20 @@
 
 #include <logging/translator.h>
 #include <tools/qbsassert.h>
+#include <tools/stringconstants.h>
 
-#include <QCoreApplication>
-#include <QDateTime>
-#include <QDir>
-#include <QFileInfo>
+#include <QtCore/qcoreapplication.h>
+#include <QtCore/qdatetime.h>
+#include <QtCore/qdir.h>
+#include <QtCore/qfileinfo.h>
+#include <QtCore/qregexp.h>
 
 #if defined(Q_OS_UNIX)
 #include <errno.h>
 #include <sys/stat.h>
 #include <unistd.h>
 #elif defined(Q_OS_WIN)
-#include <qt_windows.h>
+#include <QtCore/qt_windows.h>
 #endif
 
 namespace qbs {
@@ -84,13 +86,31 @@ QString FileInfo::completeBaseName(const QString &fp)
     return fn.mid(0, dot);
 }
 
+QString FileInfo::suffix(const QString &fp)
+{
+    QString fn = fileName(fp);
+    int dot = fn.lastIndexOf(QLatin1Char('.'));
+    if (dot < 0)
+        return fn;
+    return fn.mid(dot + 1);
+}
+
+QString FileInfo::completeSuffix(const QString &fp)
+{
+    QString fn = fileName(fp);
+    int dot = fn.indexOf(QLatin1Char('.'));
+    if (dot < 0)
+        return fn;
+    return fn.mid(dot + 1);
+}
+
 QString FileInfo::path(const QString &fp, HostOsInfo::HostOs hostOs)
 {
     if (fp.isEmpty())
-        return QString();
+        return {};
     int last = fp.lastIndexOf(QLatin1Char('/'));
     if (last < 0)
-        return QLatin1String(".");
+        return StringConstants::dot();
     QString p = QDir::cleanPath(fp.mid(0, last));
     if (p.isEmpty() || (hostOs == HostOsInfo::HostOsWindows && p.length() == 2 && p.at(0).isLetter()
             && p.at(1) == QLatin1Char(':'))) {
@@ -130,6 +150,15 @@ bool FileInfo::exists(const QString &fp)
     return FileInfo(fp).exists();
 }
 
+// Whether a path is the special "current drive path" path type,
+// which is neither truly relative nor absolute
+static bool isCurrentDrivePath(const QString &path, HostOsInfo::HostOs hostOs)
+{
+    return hostOs == HostOsInfo::HostOsWindows
+            ? path.size() == 2 && path.at(1) == QLatin1Char(':') && path.at(0).isLetter()
+            : false;
+}
+
 // from creator/src/shared/proparser/ioutils.cpp
 bool FileInfo::isAbsolute(const QString &path, HostOsInfo::HostOs hostOs)
 {
@@ -158,8 +187,7 @@ bool FileInfo::isPattern(const QString &str)
 
 bool FileInfo::isPattern(const QStringRef &str)
 {
-    for (int i = 0; i < str.size(); ++i) {
-        const QChar ch = str.at(i);
+    for (const QChar &ch : str) {
         if (ch == QLatin1Char('*') || ch == QLatin1Char('?')
                 || ch == QLatin1Char(']') || ch == QLatin1Char('[')) {
             return true;
@@ -175,17 +203,18 @@ bool FileInfo::isPattern(const QStringRef &str)
  * This function assumes that both paths are clean, that is they don't contain
  * double slashes or redundant dot parts.
  */
-QString FileInfo::resolvePath(const QString &base, const QString &rel)
+QString FileInfo::resolvePath(const QString &base, const QString &rel, HostOsInfo::HostOs hostOs)
 {
-    QBS_ASSERT(isAbsolute(base), qDebug("base: %s, rel: %s", qPrintable(base), qPrintable(rel));
-            return QString());
-    if (isAbsolute(rel))
+    QBS_ASSERT(isAbsolute(base, hostOs) && !isCurrentDrivePath(rel, hostOs),
+               qDebug("base: %s, rel: %s", qPrintable(base), qPrintable(rel));
+            return {});
+    if (isAbsolute(rel, hostOs))
         return rel;
     if (rel.size() == 1 && rel.at(0) == QLatin1Char('.'))
         return base;
     if (rel.size() == 1 && rel.at(0) == QLatin1Char('~'))
         return QDir::homePath();
-    if (rel.startsWith(QLatin1String("~/")))
+    if (rel.startsWith(StringConstants::tildeSlash()))
         return QDir::homePath() + rel.mid(1);
 
     QString r = base;
@@ -193,22 +222,23 @@ QString FileInfo::resolvePath(const QString &base, const QString &rel)
         r.chop(1);
 
     QString s = rel;
-    while (s.startsWith(QLatin1String("../"))) {
+    while (s.startsWith(QStringLiteral("../"))) {
         s.remove(0, 3);
         int idx = r.lastIndexOf(QLatin1Char('/'));
         if (idx >= 0)
             r.truncate(idx);
     }
-    if (s == QLatin1String("..")) {
+    if (s == StringConstants::dotDot()) {
         int idx = r.lastIndexOf(QLatin1Char('/'));
         if (idx >= 0)
             r.truncate(idx);
-        return r;
+        s.clear();
+    }
+    if (!s.isEmpty() || isCurrentDrivePath(r, hostOs)) {
+        r.reserve(r.length() + 1 + s.length());
+        r += QLatin1Char('/');
+        r += s;
     }
-
-    r.reserve(r.length() + 1 + s.length());
-    r += QLatin1Char('/');
-    r += s;
     return r;
 }
 
@@ -224,7 +254,14 @@ bool FileInfo::globMatches(const QRegExp &regexp, const QString &fileName)
 }
 
 #ifdef Q_OS_WIN
-static const QString win32LongPathPrefix = QStringLiteral("\\\\?\\");
+static QString prependLongPathPrefix(const QString &absolutePath)
+{
+    QString nativePath = QDir::toNativeSeparators(absolutePath);
+    if (nativePath.startsWith(QStringLiteral("\\\\")))
+        nativePath.remove(0, 1).prepend(QLatin1String("UNC"));
+    nativePath.prepend(QLatin1String("\\\\?\\"));
+    return nativePath;
+}
 #endif
 
 bool FileInfo::isFileCaseCorrect(const QString &filePath)
@@ -232,7 +269,7 @@ bool FileInfo::isFileCaseCorrect(const QString &filePath)
 #if defined(Q_OS_WIN)
     // QFileInfo::canonicalFilePath() does not return the real case of the file path on Windows.
     QFileInfo fi(filePath);
-    const QString absolute = win32LongPathPrefix + QDir::toNativeSeparators(fi.absoluteFilePath());
+    const QString absolute = prependLongPathPrefix(fi.absoluteFilePath());
     WIN32_FIND_DATA fd;
     HANDLE hFindFile = ::FindFirstFile((wchar_t*)absolute.utf16(), &fd);
     if (hFindFile == INVALID_HANDLE_VALUE)
@@ -258,15 +295,10 @@ bool FileInfo::fileExists(const QFileInfo &fi)
 
 #define z(x) reinterpret_cast<WIN32_FILE_ATTRIBUTE_DATA*>(const_cast<FileInfo::InternalStatType*>(&x))
 
-template<bool> struct CompileTimeAssert;
-template<> struct CompileTimeAssert<true> {};
-
 FileInfo::FileInfo(const QString &fileName)
 {
-    static CompileTimeAssert<
-        sizeof(FileInfo::InternalStatType) == sizeof(WIN32_FILE_ATTRIBUTE_DATA)
-            > internal_type_has_wrong_size;
-    Q_UNUSED(internal_type_has_wrong_size);
+    static_assert(sizeof(FileInfo::InternalStatType) == sizeof(WIN32_FILE_ATTRIBUTE_DATA),
+                  "FileInfo::InternalStatType has wrong size.");
 
     QString filePath = fileName;
 
@@ -274,7 +306,7 @@ FileInfo::FileInfo(const QString &fileName)
     if (!isAbsolute(filePath))
         filePath = QDir::currentPath() + QDir::separator() + filePath;
 
-    filePath = win32LongPathPrefix + QDir::toNativeSeparators(QDir::cleanPath(filePath));
+    filePath = prependLongPathPrefix(QDir::cleanPath(filePath));
     if (!GetFileAttributesEx(reinterpret_cast<const WCHAR*>(filePath.utf16()),
                              GetFileExInfoStandard, &m_stat))
     {
@@ -292,7 +324,7 @@ FileTime FileInfo::lastModified() const
 {
     const FileTime::InternalType* ft_it;
     ft_it = reinterpret_cast<const FileTime::InternalType*>(&z(m_stat)->ftLastWriteTime);
-    return FileTime(*ft_it);
+    return {*ft_it};
 }
 
 FileTime FileInfo::lastStatusChange() const
@@ -302,14 +334,14 @@ FileTime FileInfo::lastStatusChange() const
 
 bool FileInfo::isDir() const
 {
-    return z(m_stat)->dwFileAttributes & FILE_ATTRIBUTE_DIRECTORY;
+    return exists() && z(m_stat)->dwFileAttributes & FILE_ATTRIBUTE_DIRECTORY;
 }
 
 static QString resolveSymlinks(const QString &fileName)
 {
     QFileInfo fi(fileName);
     while (fi.isSymLink())
-        fi.setFile(fi.symLinkTarget());
+        fi.setFile(fi.dir(), fi.symLinkTarget());
     return fi.absoluteFilePath();
 }
 
@@ -323,23 +355,37 @@ QString applicationDirPath()
 
 FileInfo::FileInfo(const QString &fileName)
 {
-    if (stat(fileName.toLocal8Bit(), &m_stat) == -1)
+    if (stat(fileName.toLocal8Bit(), &m_stat) == -1) {
         m_stat.st_mtime = 0;
+        m_stat.st_mode = 0;
+    }
 }
 
 bool FileInfo::exists() const
 {
-    return m_stat.st_mtime != 0;
+    return m_stat.st_mode != 0;
 }
 
 FileTime FileInfo::lastModified() const
 {
+#if APPLE_STAT_TIMESPEC
+    return m_stat.st_mtimespec;
+#elif HAS_CLOCK_GETTIME
+    return m_stat.st_mtim;
+#else
     return m_stat.st_mtime;
+#endif
 }
 
 FileTime FileInfo::lastStatusChange() const
 {
+#if APPLE_STAT_TIMESPEC
+    return m_stat.st_ctimespec;
+#elif HAS_CLOCK_GETTIME
+    return m_stat.st_ctim;
+#else
     return m_stat.st_ctime;
+#endif
 }
 
 bool FileInfo::isDir() const
@@ -358,8 +404,8 @@ bool removeFileRecursion(const QFileInfo &f, QString *errorMessage)
         const QDir dir(f.absoluteFilePath());
 
         // QDir::System is needed for broken symlinks.
-        foreach (const QFileInfo &fi, dir.entryInfoList(QDir::AllEntries | QDir::NoDotAndDotDot
-                                                       | QDir::Hidden | QDir::System))
+        for (const QFileInfo &fi : dir.entryInfoList(QDir::AllEntries | QDir::NoDotAndDotDot
+                                                     | QDir::Hidden | QDir::System))
             removeFileRecursion(fi, errorMessage);
         QDir parent = f.absoluteDir();
         if (!parent.rmdir(f.fileName())) {
@@ -407,7 +453,7 @@ static QByteArray storedLinkTarget(const QString &filePath)
         if (lstat(nativeFilePath.constData(), &sb)) {
             qWarning("storedLinkTarget: lstat for %s failed with error code %d",
                      nativeFilePath.constData(), errno);
-            return QByteArray();
+            return {};
         }
 
         result.resize(sb.st_size);
@@ -415,7 +461,7 @@ static QByteArray storedLinkTarget(const QString &filePath)
         if (len < 0) {
             qWarning("storedLinkTarget: readlink for %s failed with error code %d",
                      nativeFilePath.constData(), errno);
-            return QByteArray();
+            return {};
         }
 
         if (len < sb.st_size) {
@@ -485,9 +531,9 @@ bool copyFileRecursion(const QString &srcFilePath, const QString &tgtFilePath,
     } else if (srcFileInfo.isDir()) {
         if (copyDirectoryContents) {
             QDir sourceDir(srcFilePath);
-            QStringList fileNames = sourceDir.entryList(QDir::Files | QDir::Dirs | QDir::NoDotAndDotDot
-                | QDir::Hidden | QDir::System);
-            foreach (const QString &fileName, fileNames) {
+            const QStringList fileNames = sourceDir.entryList(QDir::Files | QDir::Dirs
+                    | QDir::NoDotAndDotDot | QDir::Hidden | QDir::System);
+            for (const QString &fileName : fileNames) {
                 const QString newSrcFilePath = srcFilePath + QLatin1Char('/') + fileName;
                 const QString newTgtFilePath = tgtFilePath + QLatin1Char('/') + fileName;
                 if (!copyFileRecursion(newSrcFilePath, newTgtFilePath, preserveSymLinks,
