@@ -45,18 +45,28 @@
 #include <QtCore/qdatastream.h>
 #include <QtCore/qdebug.h>
 
-#if defined(Q_OS_UNIX)
+#if defined(Q_OS_UNIX) && !defined(__APPLE__)
 #include <time.h>
-#define BUILD_HOST_HAS_CLOCK_GETTIME (_POSIX_C_SOURCE >= 199309L)
-#ifdef __APPLE__
-#if __MAC_OS_X_VERSION_MIN_REQUIRED >= 101200
-#define HAS_CLOCK_GETTIME BUILD_HOST_HAS_CLOCK_GETTIME
-#endif // __MAC_OS_X_VERSION_MIN_REQUIRED
-#else // __APPLE__
-#define HAS_CLOCK_GETTIME BUILD_HOST_HAS_CLOCK_GETTIME
-#endif // __APPLE__
+#define HAS_CLOCK_GETTIME (_POSIX_C_SOURCE >= 199309L)
 #endif // Q_OS_UNIX
 
+#ifdef __APPLE__
+
+#if __MAC_OS_X_VERSION_MIN_REQUIRED >= 101200
+// macOS 10.12+ ships clock_gettime.
+#else
+// We implement our own clock_gettime.
+#define APPLE_CUSTOM_CLOCK_GETTIME 1
+#endif // __MAC_OS_X_VERSION_MIN_REQUIRED
+
+// Either way we have a clock_gettime in the end.
+#define HAS_CLOCK_GETTIME 1
+
+// Apple stat struct has slightly different names for time fields.
+#define APPLE_STAT_TIMESPEC 1
+
+#endif // __APPLE__
+
 namespace qbs {
 namespace Internal {
 
@@ -65,12 +75,12 @@ class QBS_AUTOTEST_EXPORT FileTime
 public:
 #if defined(Q_OS_UNIX)
 #if HAS_CLOCK_GETTIME
-    typedef timespec InternalType;
+    using InternalType = timespec;
 #else
-    typedef time_t InternalType;
+    using InternalType = time_t;
 #endif // HAS_CLOCK_GETTIME
 #elif defined(Q_OS_WIN)
-    typedef quint64 InternalType;
+    using InternalType = quint64;
 #else
 #   error unknown platform
 #endif
@@ -78,9 +88,6 @@ public:
     FileTime();
     FileTime(const InternalType &ft);
 
-    void store(PersistentPool &pool) const;
-    void load(PersistentPool &pool);
-
     bool operator<(const FileTime &rhs) const { return compare(rhs) < 0; }
     bool operator>(const FileTime &rhs) const { return compare(rhs) > 0; }
     bool operator<=(const FileTime &rhs) const { return !operator>(rhs); }
@@ -98,6 +105,15 @@ public:
 
     double asDouble() const;
 
+    template<PersistentPool::OpType opType> void completeSerializationOp(PersistentPool &pool)
+    {
+#if HAS_CLOCK_GETTIME
+        pool.serializationOp<opType>(m_fileTime.tv_sec, m_fileTime.tv_nsec);
+#else
+        pool.serializationOp<opType>(m_fileTime);
+#endif
+    }
+
 private:
     InternalType m_fileTime;
 };
