@@ -42,15 +42,13 @@
 #include "fileinfo.h"
 #include <logging/translator.h>
 #include <tools/error.h>
-#include <tools/qbsassert.h>
 
 #include <QtCore/qdir.h>
-#include <QtCore/qscopedpointer.h>
 
 namespace qbs {
 namespace Internal {
 
-static const char QBS_PERSISTENCE_MAGIC[] = "QBSPERSISTENCE_108";
+static const char QBS_PERSISTENCE_MAGIC[] = "QBSPERSISTENCE-126";
 
 NoBuildGraphError::NoBuildGraphError(const QString &filePath)
     : ErrorInfo(Tr::tr("Build graph not found for configuration '%1'. Expected location was '%2'.")
@@ -71,7 +69,7 @@ PersistentPool::~PersistentPool()
 
 void PersistentPool::load(const QString &filePath)
 {
-    QScopedPointer<QFile> file(new QFile(filePath));
+    std::unique_ptr<QFile> file(new QFile(filePath));
     if (!file->exists())
         throw NoBuildGraphError(filePath);
     if (!file->open(QFile::ReadOnly)) {
@@ -79,19 +77,19 @@ void PersistentPool::load(const QString &filePath)
                     .arg(filePath, file->errorString()));
     }
 
-    m_stream.setDevice(file.data());
+    m_stream.setDevice(file.get());
     QByteArray magic;
     m_stream >> magic;
     if (magic != QBS_PERSISTENCE_MAGIC) {
-        m_stream.setDevice(0);
+        m_stream.setDevice(nullptr);
         throw ErrorInfo(Tr::tr("Cannot use stored build graph at '%1': Incompatible file format. "
                            "Expected magic token '%2', got '%3'.")
-                    .arg(filePath, QString::fromLatin1(QBS_PERSISTENCE_MAGIC),
+                    .arg(filePath, QLatin1String(QBS_PERSISTENCE_MAGIC),
                          QString::fromLatin1(magic)));
     }
 
     m_stream >> m_headData.projectConfig;
-    file.take();
+    file.release();
     m_loadedRaw.clear();
     m_loaded.clear();
     m_storageIndices.clear();
@@ -112,16 +110,18 @@ void PersistentPool::setupWriteStream(const QString &filePath)
                         .arg(filePath));
     }
     QBS_CHECK(!QFile::exists(filePath));
-    QScopedPointer<QFile> file(new QFile(filePath));
+    std::unique_ptr<QFile> file(new QFile(filePath));
     if (!file->open(QFile::WriteOnly)) {
         throw ErrorInfo(Tr::tr("Failure storing build graph: "
                 "Cannot open file '%1' for writing: %2").arg(filePath, file->errorString()));
     }
 
-    m_stream.setDevice(file.take());
+    m_stream.setDevice(file.release());
     m_stream << QByteArray(qstrlen(QBS_PERSISTENCE_MAGIC), 0) << m_headData.projectConfig;
     m_lastStoredObjectId = 0;
     m_lastStoredStringId = 0;
+    m_lastStoredEnvId = 0;
+    m_lastStoredStringListId = 0;
 }
 
 void PersistentPool::finalizeWriteStream()
@@ -132,7 +132,7 @@ void PersistentPool::finalizeWriteStream()
     m_stream << QByteArray(QBS_PERSISTENCE_MAGIC);
     if (m_stream.status() != QDataStream::Ok)
         throw ErrorInfo(Tr::tr("Failure serializing build graph."));
-    QFile * const file = static_cast<QFile *>(m_stream.device());
+    const auto file = static_cast<QFile *>(m_stream.device());
     if (!file->flush()) {
         file->close();
         file->remove();
@@ -143,25 +143,10 @@ void PersistentPool::finalizeWriteStream()
 void PersistentPool::closeStream()
 {
     delete m_stream.device();
-    m_stream.setDevice(0);
+    m_stream.setDevice(nullptr);
 }
 
-void PersistentPool::storePersistentObject(const PersistentObject *object)
-{
-    if (!object) {
-        m_stream << -1;
-        return;
-    }
-    PersistentObjectId id = m_storageIndices.value(object, -1);
-    if (id < 0) {
-        id = m_lastStoredObjectId++;
-        m_storageIndices.insert(object, id);
-        m_stream << id;
-        object->store(*this);
-    } else {
-        m_stream << id;
-    }
-}
+
 
 void PersistentPool::storeVariant(const QVariant &variant)
 {
@@ -169,7 +154,7 @@ void PersistentPool::storeVariant(const QVariant &variant)
     m_stream << type;
     switch (type) {
     case QMetaType::QString:
-        storeString(variant.toString());
+        store(variant.toString());
         break;
     case QMetaType::QStringList:
         store(variant.toStringList());
@@ -191,7 +176,7 @@ QVariant PersistentPool::loadVariant()
     QVariant value;
     switch (type) {
     case QMetaType::QString:
-        value = idLoadString();
+        value = load<QString>();
         break;
     case QMetaType::QStringList:
         value = load<QStringList>();
@@ -216,50 +201,48 @@ void PersistentPool::clear()
     m_inverseStringStorage.clear();
 }
 
-const int StringNotFoundId = -1;
-const int NullStringId = -2;
-
-void PersistentPool::storeString(const QString &t)
+void PersistentPool::doLoadValue(QString &s)
 {
-    if (t.isNull()) {
-        m_stream << NullStringId;
-        return;
-    }
-
-    int id = m_inverseStringStorage.value(t, StringNotFoundId);
-    if (id < 0) {
-        id = m_lastStoredStringId++;
-        m_inverseStringStorage.insert(t, id);
-        m_stream << id << t;
-    } else {
-        m_stream << id;
-    }
+    m_stream >> s;
 }
 
-QString PersistentPool::loadString(int id)
+void PersistentPool::doLoadValue(QStringList &l)
 {
-    if (id == NullStringId)
-        return QString();
-
-    QBS_CHECK(id >= 0);
-
-    if (id >= static_cast<int>(m_stringStorage.size())) {
-        QString s;
-        m_stream >> s;
-        m_stringStorage.resize(id + 1);
-        m_stringStorage[id] = s;
-        return s;
-    }
-
-    return m_stringStorage.at(id);
+    int size;
+    m_stream >> size;
+    for (int i = 0; i < size; ++i)
+        l << load<QString>();
 }
 
-QString PersistentPool::idLoadString()
+void PersistentPool::doLoadValue(QProcessEnvironment &env)
 {
-    int id;
-    m_stream >> id;
-    return loadString(id);
+    const QStringList keys = load<QStringList>();
+    for (const QString &key : keys)
+        env.insert(key, load<QString>());
 }
 
+void PersistentPool::doStoreValue(const QString &s)
+{
+    m_stream << s;
+}
+
+void PersistentPool::doStoreValue(const QStringList &l)
+{
+    m_stream << l.size();
+    for (const QString &s : l)
+        store(s);
+}
+
+void PersistentPool::doStoreValue(const QProcessEnvironment &env)
+{
+    const QStringList &keys = env.keys();
+    store(keys);
+    for (const QString &key : keys)
+        store(env.value(key));
+}
+
+const PersistentPool::PersistentObjectId PersistentPool::ValueNotFoundId;
+const PersistentPool::PersistentObjectId PersistentPool::EmptyValueId;
+
 } // namespace Internal
 } // namespace qbs
