@@ -39,6 +39,7 @@
 #include "profile.h"
 #include "qbsassert.h"
 #include "settings.h"
+#include "stringconstants.h"
 
 #include <logging/translator.h>
 #include <tools/error.h>
@@ -73,8 +74,8 @@ Profile::Profile(const QString &name, Settings *settings, const QVariantMap &pro
 
 bool Profile::exists() const
 {
-    return m_name == fallbackName() || !m_values.isEmpty()
-            || !m_settings->allKeysWithPrefix(profileKey()).isEmpty();
+    return m_name == fallbackName() || !m_values.empty()
+            || !m_settings->allKeysWithPrefix(profileKey(), Settings::allScopes()).empty();
 }
 
 /*!
@@ -87,7 +88,7 @@ QVariant Profile::value(const QString &key, const QVariant &defaultValue, ErrorI
     } catch (const ErrorInfo &e) {
         if (error)
             *error = e;
-        return QVariant();
+        return {};
     }
 }
 
@@ -131,7 +132,7 @@ QStringList Profile::allKeys(KeySelection selection, ErrorInfo *error) const
     } catch (const ErrorInfo &e) {
         if (error)
             *error = e;
-        return QStringList();
+        return {};
     }
 }
 
@@ -180,12 +181,12 @@ QString Profile::cleanName(const QString &name)
 
 QString Profile::profileKey() const
 {
-    return QLatin1String("profiles.") + m_name;
+    return Internal::StringConstants::profilesSettingsPrefix() + m_name;
 }
 
 QString Profile::baseProfileKey()
 {
-    return QLatin1String("baseProfile");
+    return Internal::StringConstants::baseProfileProperty();
 }
 
 void Profile::checkBaseProfileExistence(const Profile &baseProfile) const
@@ -199,7 +200,7 @@ QVariant Profile::localValue(const QString &key) const
 {
     QVariant val = m_values.value(key);
     if (!val.isValid())
-        val = m_settings->value(fullyQualifiedKey(key));
+        val = m_settings->value(fullyQualifiedKey(key), Settings::allScopes());
     return val;
 }
 
@@ -228,8 +229,8 @@ QStringList Profile::allKeysInternal(Profile::KeySelection selection,
 {
     extendAndCheckProfileChain(profileChain);
     QStringList keys = m_values.keys();
-    if (keys.isEmpty())
-        keys = m_settings->allKeysWithPrefix(profileKey());
+    if (keys.empty())
+        keys = m_settings->allKeysWithPrefix(profileKey(), Settings::allScopes());
     if (selection == KeySelectionNonRecursive)
         return keys;
     const QString baseProfileName = baseProfile();
