@@ -63,29 +63,31 @@ void Benchmarker::benchmark()
 {
     rememberCurrentRepoState();
     runProcess(QStringList() << "git" << "checkout" << m_oldCommit, m_qbsRepo);
-    const QString oldQbsBuildDir = m_baseOutputDir.path() + "/old-qbs-build";
+    const QString oldQbsBuildDir = m_baseOutputDir.path() + "/qbs-build." + m_oldCommit;
     std::cout << "Building from old repo state..." << std::endl;
     buildQbs(oldQbsBuildDir);
     runProcess(QStringList() << "git" << "checkout" << m_newCommit, m_qbsRepo);
-    const QString newQbsBuildDir = m_baseOutputDir.path() + "/new-qbs-build";
+    const QString newQbsBuildDir = m_baseOutputDir.path() + "/qbs-build." + m_newCommit;
     std::cout << "Building from new repo state..." << std::endl;
     buildQbs(newQbsBuildDir);
     std::cout << "Now running valgrind. This can take a while." << std::endl;
 
     ValgrindRunner oldDataRetriever(m_activities, m_testProject, oldQbsBuildDir,
-                                    m_baseOutputDir.path() + "/old-stuff");
+                                    m_baseOutputDir.path() + "/benchmark-data." + m_oldCommit);
     ValgrindRunner newDataRetriever(m_activities, m_testProject, newQbsBuildDir,
-                                    m_baseOutputDir.path() + "/new-stuff");
+                                    m_baseOutputDir.path() + "/benchmark-data." + m_newCommit);
     QFuture<void> oldFuture = QtConcurrent::run(&oldDataRetriever, &ValgrindRunner::run);
     QFuture<void> newFuture = QtConcurrent::run(&newDataRetriever, &ValgrindRunner::run);
     oldFuture.waitForFinished();
-    foreach (const ValgrindResult &valgrindResult, oldDataRetriever.results()) {
+    const auto oldValgrindResults = oldDataRetriever.results();
+    for (const ValgrindResult &valgrindResult : oldValgrindResults) {
         BenchmarkResult &benchmarkResult = m_results[valgrindResult.activity];
         benchmarkResult.oldInstructionCount = valgrindResult.instructionCount;
         benchmarkResult.oldPeakMemoryUsage = valgrindResult.peakMemoryUsage;
     }
     newFuture.waitForFinished();
-    foreach (const ValgrindResult &valgrindResult, newDataRetriever.results()) {
+    const auto newValgrindResults = newDataRetriever.results();
+    for (const ValgrindResult &valgrindResult : newValgrindResults) {
         BenchmarkResult &benchmarkResult = m_results[valgrindResult.activity];
         benchmarkResult.newInstructionCount = valgrindResult.instructionCount;
         benchmarkResult.newPeakMemoryUsage = valgrindResult.peakMemoryUsage;
@@ -114,8 +116,9 @@ void Benchmarker::rememberCurrentRepoState()
 void Benchmarker::buildQbs(const QString &buildDir) const
 {
     if (!QDir::root().mkpath(buildDir))
-        throw Exception(QString::fromLatin1("Failed to create directory '%1'.").arg(buildDir));
-    runProcess(QStringList() << "qmake" << m_qbsRepo + "/qbs.pro", buildDir);
+        throw Exception(QStringLiteral("Failed to create directory '%1'.").arg(buildDir));
+    runProcess(QStringList() << "qmake" << "CONFIG+=force_debug_info"
+               << (m_qbsRepo + "/qbs.pro"), buildDir);
     runProcess(QStringList() << "make" << "-s", buildDir);
 }
 
